//************************************************************************
//*                          BipGDI Win V0.9.3                           *
//*----------------------------------------------------------------------*
//*                 BipGDI FUNCTIONS for Microsoft Windows               *
//*                                                                      *
//* Compilation options :                                                *
//* add comctl32.lib in Project/Settings/Link/Objet library modules      *
//*                                                                      *
//*                                                         P. Basset    *
//************************************************************************

#pragma warning (disable:4786)	// disable warning "identifier was truncated to '255' characters" when using std::map


#include <windows.h>
#include <string>
#include <map>
#include <fstream>
#include <sstream>
#include <iomanip>			// for std::setbase
#include <time.h>			// for error file log
#include <commctrl.h>		// for listview and status bar

#include "BipGDI.h"
#include "BipDATE.h"
#include "BipNUM.h"



	using namespace std;



	// Global Variables : can be modify
	string BG_Language				= "FR";			// default language

	string sBG_DefFontName			= "Courier New";// Default font name
	int iBG_DefFontHeight			= 10;			// default font height

	const int BG_MAX_MENU		    = 100;			// max number of menu
													// (BG_MAX_MENU^2 must fit in INT)

	const string BG_POPUP_NAME		= "BGPOPUP";	// name of the BG_InputBox window
	const string BG_DG_CLASS_NAME	= "BGDATAGRID";	// name of the DataGrid Class

	bool BG_DISPLAY_ERROR			= true;			// if true then display error message

	int iBG_SplitBarSize			= 4;			// default size for the split bar



	// #### Global Variables #### : **** DON'T MOFIFY ****
	const string BG_PLATEFORM		= "WIN";		// plateform of BipGDI
	const string BG_VERSION			= "0.9.3";		// version of BipGDI

	const int BG_ITEM_NUM			= 101;			// first number for form item
													// 100 is reserved for status bar
	const int BG_MENU_NUM			= 40001;		// first number for menu 



	HINSTANCE hBGInstance;							// instance of the program
	HMENU hBGMenu;									// handle for the menu

	BG_SCROLL_BAR BGScrollBar;						// size of scroll bar
//	string sBGExtractCSV [BG_MAX_EXTRACT];			// string for BG_ExtractCSV function
	BG_INPUTBOX BGInputBox;							// structure for Input Box
	BG_STRUCT_ERROR BGError;						// structure for BipGDI error
	BG_STRUCT_ERROR BGErrorSave;					// structure for BipGDI error
	string BGCurWindow;								// current active window
	string BGMainWindow;							// internal name of the main window
	string BGCurForm;								// current active Form
	string sBGItemOldValue;							// item value before modify
	bool bBGItemOldValue;							// item value before modify


	bool BG_RESETBIPGDI_DONE		= false;		// if false then run BG_ResetBipGDI

	PAINTSTRUCT psBGps;								// paint structure
	HDC hBGdc;										// draw context

	// menu declaration
	typedef map <int, BG_STRUCT_MENU> BGMenuDef;	// map for menu info
	BGMenuDef BGMenu;								// declare menu map
	typedef BGMenuDef::iterator BGMenuItDef;		// iterator for menu map

	// Window declaration
	typedef map <string, BG_STRUCT_WINDOW> BGWindowDef;// map for window
	BGWindowDef BGWindow;							// declare window map
	typedef BGWindowDef::iterator BGWindowItDef;	// iterator for window map

	// Form Item declaration
	typedef map <string, BG_STRUCT_FORMITEM> BGItemDef;// map for form item
	BGItemDef BGItem;								// declare form item map
	typedef BGItemDef::iterator BGItemItDef;		// iterator for form item map
	// #### End of Global Variables #### : **** DON'T MOFIFY ****





//-----------------------------------------------------------------
//!    BG_WindowMainCreate : Create main window                   !
//-----------------------------------------------------------------
//! sBGWindowName : internal name of the main window              !
//! nCmsShow : show state                                         !
//! wndproc : window procedure                                    !
//! sBGParam : parameters (default = "")                          !
//! iBGLeft : left position of the window in pixel (default = 0)  !
//! iBGTop : top position of the window in pixel (default = 0)    !
//! iBGWidth : width of the window in pixel (default = 0)         !
//! iBGHeight : height of the window in pixel (default = 0)       !
//!																  !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
//!	for a full screen window, set iLeft,iTop,iWidth,iHeight to 0  !
//-----------------------------------------------------------------
int BG_WindowMainCreate (const string sBGWindowName, const int nCmdShow,
						const WNDPROC wndproc, const string sBGParam,
						const int iBGLeft, const int iBGTop, const int iBGWidth,
						const int iBGHeight)
						// sBGParam = "", iBGLeft = 0, iBGTop = 0, iBGWidth = 0, iBGHeight = 0
{
	HWND hCurWnd;
	WNDCLASSEX wcex;
	HBRUSH hBrush;
	string sString;
	int iBackColor;
	int iError;
	BG_STRUCT_WINDOW TmpWindow;
	BGWindowItDef BGWindowIt;
	string sMainWindow;


	// for error management
	BG_ErrorSet (sBGWindowName, "BG_WindowMainCreate");

	// reset BipGDI before use and only one time 
	if (!BG_RESETBIPGDI_DONE)
		BG_ResetBipGDIIntl ();

	// save window name into string
	sMainWindow = sBGWindowName;

	// verify window's name
	if (sBGWindowName == "")
	{
		BG_ErrorDisplay (BG_ERR_BADWINDNAME);
		return BG_ERR_BADWINDNAME;
	}

	// verify if window's name contain "." character
	iError = sBGWindowName.find ('.', 0);
	if (iError >= 0)
	{
		BG_ErrorDisplay (BG_ERR_NOPOINT);
		return BG_ERR_NOPOINT;
	}


	// verify if main windows is already open
	if (BGMainWindow != "")
	{
		BG_ErrorDisplay (BG_ERR_REOPEN);
		return BG_ERR_REOPEN;
	}

	// search background color parameter
	sString = BG_StringParamGetInt (BG_PRM_BACKCOLOR, sBGParam);
	if (sString == "")
		sString = "white";
	iBackColor = BG_ColorNumGet (sString);

	// create background brush
	hBrush = CreateSolidBrush (iBackColor);
	if (hBrush == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32CREATESBRUSH );
		return BG_ERR_W32CREATESBRUSH ;
	}

	// register the window class
	wcex.cbSize			= sizeof (WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= wndproc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hBGInstance;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor (NULL, IDC_ARROW);
	wcex.hbrBackground	= hBrush;
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= sMainWindow.data ();
	wcex.hIconSm		= NULL;

	iError = RegisterClassEx (&wcex);
	if (iError == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32REGCLASS);
		return BG_ERR_W32REGCLASS;
	}


	// create window
	hCurWnd = CreateWindowEx (WS_EX_LEFT, sMainWindow.data (), sMainWindow.data (),
								WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT,
								CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hBGInstance,
								NULL);
    if (hCurWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32CREATEWINDOW);
		return BG_ERR_W32CREATEWINDOW;
	}


	// store values in window structure
	BGMainWindow = sBGWindowName;
	TmpWindow = BG_WindowNullGet ();
	TmpWindow.hBGWnd = hCurWnd;
	TmpWindow.iBackColor = iBackColor;

	BGWindow [BGMainWindow] = TmpWindow;


	// set window title
	sString = BG_StringParamGetInt (BG_PRM_NAME, sBGParam);
	if (sString != "")
	{
		iError = BG_WindowTitleSet (BGMainWindow, sString);
		if (iError != 0)
			return iError;

		// store values in window structure
		TmpWindow.sTitle = sString;
		BGWindow [BGMainWindow] = TmpWindow;
	}

	// set current window
	BGCurWindow = BGMainWindow;

	// set the size of the main window and display it
	iError = BG_WindowMove (BGMainWindow, iBGLeft, iBGTop, iBGWidth, iBGHeight);
	if (iError != 0)
		return iError;

	return 0;
}




//-----------------------------------------------------------------
//!    BG_WindowChildCreate : Create Child (frame) Window         !
//-----------------------------------------------------------------
//! sBGChildName : name of the child window                       !
//! childwndproc : window procedure for the child window          !
//!              default value is false                           !
//! sBGParam : parameters (default = "")                          !
//! iBGLeft : left position of the window (in pixel)              !
//! iBGTop : top position of the window (in pixel)                !
//! iBGWidth : width of the window (in pixel)                     !
//! iBGHeight : height of the window (in pixel)                   !
//!    default value is 0 (= full screen)                         !
//!																  !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_WindowChildCreate (const string sBGChildName, const WNDPROC childwndproc,
						 const string sBGParam, const int iBGLeft, const int iBGTop,
						 const int iBGWidth, const int iBGHeight)
						// sBGParam = "", iBGLeft = 0, iBGTop = 0, iBGWidth = 0,
						// iBGHeight = 0
{

	HWND hCurWnd;
	WNDCLASSEX wcex;
	HBRUSH hBrush;
	int iBackColor;
	bool bEdge;
	string sString;
	BG_STRUCT_WINDOW TmpWindow;
	BG_STRUCT_WINDOW MainWindow;
	BGWindowItDef BGWindowIt;
	DWORD dwBorder;
	int iError;


	// for error management
	BG_ErrorSet (sBGChildName, "BG_WindowChildCreate");

	// verify window's name
	if (sBGChildName == "")
	{
		BG_ErrorDisplay (BG_ERR_BADWINDNAME);
		return BG_ERR_BADWINDNAME;
	}

	// window is already open
	TmpWindow = BG_WindowGet (sBGChildName);
	if (TmpWindow.hBGWnd != NULL)
	{
		BG_ErrorDisplay (BG_ERR_REOPEN);
		return BG_ERR_REOPEN;
	}

	// main window not open
	MainWindow = BG_WindowGet (BGMainWindow);
	if (MainWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_MAINNOTOPEN);
		return BG_ERR_MAINNOTOPEN;
	}

	// verify if window's name contain "." character
	iError = sBGChildName.find ('.', 0);
	if (iError >= 0)
	{
		BG_ErrorDisplay (BG_ERR_NOPOINT);
		return BG_ERR_NOPOINT;
	}

	// search background color parameter, dafault is white
	sString = BG_StringParamGetInt (BG_PRM_BACKCOLOR, sBGParam);
	if (sString == "")
		sString = "white";
	iBackColor = BG_ColorNumGet (sString);

	// create background brush
	hBrush = CreateSolidBrush (iBackColor);
	if (hBrush == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32CREATESBRUSH );
		return BG_ERR_W32CREATESBRUSH ;
	}

	// Register the window class
	wcex.cbSize			= sizeof (WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= childwndproc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hBGInstance;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor (NULL, IDC_ARROW);
	wcex.hbrBackground	= hBrush;
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= sBGChildName.data ();
	wcex.hIconSm		= NULL;

	iError = RegisterClassEx (&wcex);
	if (iError == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32REGCLASS);
		return BG_ERR_W32REGCLASS;
	}


	// search border parameter
	sString = BG_StringParamGetInt (BG_PRM_EDGE, sBGParam);
	bEdge = false;
	if (BG_StringIsTrue (sString))
		bEdge = true;

	// set border
	if (bEdge)
		dwBorder = WS_CHILD | WS_BORDER;
	else
		dwBorder = WS_CHILD;

	// create window
    hCurWnd = CreateWindowEx (WS_EX_LEFT, sBGChildName.data (), "", dwBorder,
								CW_USEDEFAULT, CW_USEDEFAULT, 0, 0,
								MainWindow.hBGWnd, NULL, hBGInstance, NULL);
	if (hCurWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32CREATEWINDOW);
		return BG_ERR_W32CREATEWINDOW;
	}

	// store values in window structure
	TmpWindow = BG_WindowNullGet ();
	TmpWindow.sTitle = sBGChildName;
	TmpWindow.hBGWnd = hCurWnd;
	TmpWindow.iBackColor = iBackColor;
	BGWindow [sBGChildName] = TmpWindow;

	// set current window
	BGCurWindow = sBGChildName;

	// set the size of the child window and display it
	iError = BG_WindowMove (sBGChildName, iBGLeft, iBGTop, iBGWidth, iBGHeight);
	if (iError != 0)
		return iError;

	return 0;
}



//-----------------------------------------------------------------
//!    BG_FormCreate : Create a form Window                       !
//-----------------------------------------------------------------
//! sBGFormName : name of the form                                !
//! sBGParam : parameters                                         !
//! iBGLeft : left position of the window (in pixel)              !
//! iBGTop : top position of the window (in pixel)                !
//! iBGWidth : width of the window (in pixel)                     !
//! iBGHeight : height of the window (in pixel)                   !
//! pfBG_FormProc : pointer to procedure user function            !
//!																  !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_FormCreate (const string sBGFormName, const string sBGParam,
				   const int iBGLeft, const int iBGTop, const int iBGWidth,
				   const int iBGHeight, pfBG_FormProc BG_FormProc)
					// = NULL
{
	int iBackColor;
	string sString;
	string sFormName;
	HGLOBAL hgMem;
	LPDLGTEMPLATE lpDlgtplt;
	LPWORD lpWord;
	LPWSTR lpWstr;
	string sDlgFont;
	int iNbChar;
	DWORD dwStyle;
	bool bStatusBar;
	bool bModal;
	BG_STRUCT_WINDOW TmpWindow;
	BG_STRUCT_WINDOW MainWindow;
	BGWindowItDef BGWindowIt;
	int iError;
	INT_PTR ipError;



	// for error management
	BG_ErrorSet (sBGFormName, "BG_FormCreate");

	// initialise variables
	BGCurForm = sBGFormName;


	// verify window's name
	if (sBGFormName == "")
	{
		BG_ErrorDisplay (BG_ERR_BADWINDNAME);
		return BG_ERR_BADWINDNAME;
	}

	// window is already open
	TmpWindow = BG_WindowGet (sBGFormName);
	if (TmpWindow.hBGWnd != NULL)
	{
		BG_ErrorDisplay (BG_ERR_REOPEN);
		return BG_ERR_REOPEN;
	}

	// verify if window's name contain "." character
	iError = sBGFormName.find ('.', 0);
	if (iError >= 0)
	{
		BG_ErrorDisplay (BG_ERR_NOPOINT);
		return BG_ERR_NOPOINT;
	}

	// get main window info
	MainWindow = BG_WindowGet (BGMainWindow);
	if (MainWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_MAINNOTOPEN);
		return BG_ERR_MAINNOTOPEN;
	}

	// search background color parameter
	sString = BG_StringParamGetInt (BG_PRM_BACKCOLOR, sBGParam);
	if (sString == "")
		sString = "white";
	iBackColor = BG_ColorNumGet (sString);

	// set style
	dwStyle = WS_VISIBLE;
	
	// search border parameter
	sString = BG_StringParamGetInt (BG_PRM_EDGE, sBGParam);
	if (sString == "false")
		dwStyle = dwStyle | WS_CHILD;
	else
		dwStyle = dwStyle | DS_CENTER | DS_MODALFRAME | WS_TILEDWINDOW | WS_POPUP;


	// search form title
	sFormName = BG_StringParamGetInt (BG_PRM_NAME, sBGParam);
	if (sFormName == "")
		sFormName = "BipGDI Form";

	// search status bar state
	sString = BG_StringParamGetInt (BG_PRM_STATUSBAR, sBGParam);
	if (BG_StringIsFalse (sString))
		bStatusBar = false;
	else
		bStatusBar = true;

	// search modal state
	sString = BG_StringParamGetInt (BG_PRM_MODAL, sBGParam);
	if (BG_StringIsTrue (sString))
		bModal = true;
	else
		bModal = false;


	// *** create dynamic dialog box ***

	// allocate memory for inputbox template
	hgMem = GlobalAlloc (GMEM_ZEROINIT, 1024);
	if (hgMem == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32GLOBALALLOC);
		return BG_ERR_W32GLOBALALLOC;
	}

	// lock template memory
	lpDlgtplt = (LPDLGTEMPLATE) GlobalLock (hgMem);
	if (lpDlgtplt == NULL)
	{
		// free memory, no error verify
		GlobalFree (hgMem);
		
		BG_ErrorDisplay (BG_ERR_W32GLOBALLOCK);
		return BG_ERR_W32GLOBALLOCK;
	}

	// description of dialog box
	lpDlgtplt->style = dwStyle;
	lpDlgtplt->dwExtendedStyle = 0;
	lpDlgtplt->cdit = 0;
	lpDlgtplt->x = MulDiv(iBGTop, 4, LOWORD (GetDialogBaseUnits ()));
	lpDlgtplt->y = MulDiv(iBGLeft, 8, HIWORD (GetDialogBaseUnits ()));
	lpDlgtplt->cx = MulDiv(iBGWidth, 4, LOWORD (GetDialogBaseUnits ()));
	lpDlgtplt->cy = MulDiv(iBGHeight, 8, HIWORD (GetDialogBaseUnits ()));

	lpWord = (LPWORD) (lpDlgtplt + 1);
	*lpWord++ = 0;   // no menu
	*lpWord++ = 0;   // predefined dialog box class

	// dialog box title
	lpWstr = (LPWSTR) lpWord;
	iNbChar = 1 + MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, sFormName.c_str(),
						 sFormName.length(), lpWstr, sFormName.length());
	lpWord += iNbChar;

	// dialog box font
	*lpWord++ = 10;   // font size

	sDlgFont = "Arial";
	lpWstr = (LPWSTR) lpWord;
	iNbChar = 1 + MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, sDlgFont.c_str(),
						 sDlgFont.length(), lpWstr, sDlgFont.length());
	lpWord += iNbChar;


	// unlock template memory
	iError = GlobalUnlock (hgMem);
	if (iError != 0)
//	modify because of Win98
//	if (iError == 0 && GetLastError () != NO_ERROR)
	{
		// free memory, no error verify
		GlobalFree (hgMem);
		
		return BG_ERR_W32GLOBALUNLOCK;
	}


	// store values in window structure
	TmpWindow = BG_WindowNullGet ();
	TmpWindow.sTitle = sBGFormName;
//	TmpWindow.hBGWnd = ;	// save later
	TmpWindow.iBackColor = iBackColor;
	TmpWindow.bStatusBar = bStatusBar;
	TmpWindow.bModal = bModal;
	TmpWindow.pfFormProc = BG_FormProc;

	BGWindow [sBGFormName] = TmpWindow;



	if (TmpWindow.bModal)
	{
		// create modal dialog window
		ipError = DialogBoxIndirect (hBGInstance, (LPDLGTEMPLATE) hgMem,
								MainWindow.hBGWnd, (DLGPROC) BG_FormProcIntl);
		if (ipError == 0 || ipError == -1)
		{
			// free memory, no error verify
			GlobalFree(hgMem);

			return BG_ERR_W32DIALOGBOXIND;
		}
	}
	else
	{
		// create modeless dialog window
		TmpWindow.hBGWnd = CreateDialogIndirect (hBGInstance,
								(LPDLGTEMPLATE) hgMem, MainWindow.hBGWnd,
								(DLGPROC) BG_FormProcIntl);
		if (TmpWindow.hBGWnd == NULL)
		{
			// free memory, no error verify
			GlobalFree(hgMem);

			return BG_ERR_W32DIALOGBOXIND;
		}

		// store handle in window structure
		BGWindow [sBGFormName] = TmpWindow;
	}


	// free template
	hgMem = GlobalFree (hgMem);
	if (hgMem != NULL)
	{
		return BG_ERR_W32GLOBALFREE;
	}


	return 0;

}



//-----------------------------------------------------------------
//!    BG_FormClose : Close a form Window                         !
//-----------------------------------------------------------------
//! sBGFormName : name of the form                                !
//!																  !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_FormClose (const string sBGFormName)
{
	string sMapForm;
	string sFormName;
	BG_STRUCT_WINDOW TmpWindow;
	BG_STRUCT_WINDOW MainWindow;
	BGItemItDef BGItemIt;
	int iFound;


	// for error management
	BG_ErrorSet (sBGFormName, "BG_FormClose");

	// look for the window number
	TmpWindow = BG_WindowGet (sBGFormName);
	if (TmpWindow.sTitle == "")
	{
		BG_ErrorDisplay (BG_ERR_NOTOPEN);
		return BG_ERR_NOTOPEN;
	}

	sFormName = sBGFormName + ".";

	// delete item for selected form, one by one
	iFound = 1;
	while (iFound == 1)
	{
		iFound = 0;
		for (BGItemIt = BGItem.begin(); BGItemIt != BGItem.end(); BGItemIt++)
		{
			sMapForm = BGItemIt->first;
			if (sMapForm.substr (0, sFormName.length()) == sFormName)
			{
				BGItem.erase (sMapForm);
				iFound = 1;
				break;
			}
		}
	}

	// delete window structure
	BGWindow.erase (sBGFormName);

	return 0;

}



//-----------------------------------------------------------------
//!    BG_FormCreateItem : create one item into a open form       !
//-----------------------------------------------------------------
//! hBGCurWnd : handle of the current form window                 !
//! sBGFormName : name of the form                                !
//! sBGItemName : name of the item                                !
//!																  !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
//! use by BG_FormProcIntl                                        !
//-----------------------------------------------------------------
int BG_FormCreateItem (const HWND hBGCurWnd, const string sBGFormName,
					   const string sBGItemName)
{
	HWND hItem;
	int iError;
	int iId;
	int iTemp;
	string sString;
	DWORD dwStyle;
	BG_STRUCT_FORMITEM TmpItem;


	// for error management
	BG_ErrorSave ();
	BG_ErrorSet (sBGFormName + "." + sBGItemName, "BG_FormCreateItem");

	// get item info
	TmpItem = BG_FormItemGet (sBGFormName, sBGItemName);
	if (TmpItem.sType == "")
	{
		BG_ErrorDisplay (BG_ERR_FORMITEMNAMEBAD);
		return BG_ERR_FORMITEMNAMEBAD;
	}

	// verify if window's name contain "." character
	iError = sBGItemName.find ('.', 0);
	if (iError >= 0)
	{
		BG_ErrorDisplay (BG_ERR_NOPOINT);
		return BG_ERR_NOPOINT;
	}


	// number of the item
	iId = TmpItem.iOrder + BG_ITEM_NUM;
	

	// create STATIC item
	if (TmpItem.sType == BG_ITEM_STATIC)
	{
		// define style of the item
		dwStyle = WS_CHILD | SS_OWNERDRAW;
		if (!TmpItem.bVisible)
			dwStyle = dwStyle | WS_DISABLED;
		else
			dwStyle = dwStyle | WS_VISIBLE;

		// create static text
		hItem = CreateWindowEx (0, "STATIC", sBGItemName.c_str (), dwStyle,
			TmpItem.iLeft, TmpItem.iTop, TmpItem.iWidth, TmpItem.iHeight,
			hBGCurWnd, NULL, hBGInstance, NULL);
		if (hItem == NULL)
		{
			BG_ErrorDisplay (BG_ERR_W32CREATEWINDOW);
			return BG_ERR_W32CREATEWINDOW;
		}

		// save item info
		TmpItem.hHandle = hItem;
		TmpItem.iId = iId;
		BGItem [sBGFormName + "." + sBGItemName] = TmpItem;

		// restore error message
		BG_ErrorRestore ();

		return 0;
	}


	// create RECT item (rectangle)
	if (TmpItem.sType == BG_ITEM_RECT)
	{
		// define style of the item
		dwStyle = WS_CHILD | SS_OWNERDRAW;
		if (!TmpItem.bVisible)
			dwStyle = dwStyle | WS_DISABLED;
		else
			dwStyle = dwStyle | WS_VISIBLE;

		// create static control
		hItem = CreateWindowEx (0, "STATIC", sBGItemName.c_str (), dwStyle,
			TmpItem.iLeft, TmpItem.iTop, TmpItem.iWidth, TmpItem.iHeight,
			hBGCurWnd, NULL, hBGInstance, NULL);
		if (hItem == NULL)
		{
			BG_ErrorDisplay (BG_ERR_W32CREATEWINDOW);
			return BG_ERR_W32CREATEWINDOW;
		}

		// save item info
		TmpItem.hHandle = hItem;
		TmpItem.iId = iId;
		BGItem [sBGFormName + "." + sBGItemName] = TmpItem;

		// restore error message
		BG_ErrorRestore ();

		return 0;
	}


	// create system EDITBOX item
	if (TmpItem.sType == BG_ITEM_EDIT)
	{
		// define style of the item
		dwStyle = WS_CHILD | WS_TABSTOP | ES_AUTOHSCROLL;
		if (!TmpItem.bVisible)
			dwStyle = dwStyle | WS_DISABLED;
		else
			dwStyle = dwStyle | WS_VISIBLE;

		// right align if item is currency or date
		if (TmpItem.sRetType == BG_RET_DATE || TmpItem.sRetType == BG_RET_CURRENCY)
				dwStyle = dwStyle | ES_RIGHT;

		// create edit box
		hItem = CreateWindowEx (WS_EX_OVERLAPPEDWINDOW, "EDIT", "", dwStyle,
					TmpItem.iLeft, TmpItem.iTop, TmpItem.iWidth, TmpItem.iHeight,
					hBGCurWnd, (HMENU) iId, hBGInstance, NULL);
		if (hItem == NULL)
		{
			BG_ErrorDisplay (BG_ERR_W32CREATEWINDOW);
			return BG_ERR_W32CREATEWINDOW;
		}

		// set max number of char
		if (TmpItem.iLength > 0 || TmpItem.iLength < 32766)
			SendMessage (hItem, EM_LIMITTEXT, TmpItem.iLength, 0);
		
		// set edit box default text
		iError = SetWindowText (hItem, TmpItem.sValue.c_str());
		if (iError == 0)
		{
			BG_ErrorDisplay (BG_ERR_W32SETWINDOWTEXT);
			return BG_ERR_W32SETWINDOWTEXT;
		}

		// save item handle and id
		TmpItem.hHandle = hItem;
		TmpItem.iId = iId;
		BGItem [sBGFormName + "." + sBGItemName] = TmpItem;

		// enable editbox
		if (!TmpItem.bEnable)
			EnableWindow (hItem, false);

		// restore error message
		BG_ErrorRestore ();

		return 0;
	}


	// create system BUTTON item
	if (TmpItem.sType == BG_ITEM_BUTTONSYS)
	{
		// define style of the item
		dwStyle = WS_CHILD | BS_PUSHBUTTON | WS_TABSTOP | BS_NOTIFY;
		if (!TmpItem.bVisible)
			dwStyle = dwStyle | WS_DISABLED;
		else
			dwStyle = dwStyle | WS_VISIBLE;

		// create Button item
		hItem = CreateWindowEx (0, "BUTTON", sBGItemName.c_str (), dwStyle,
					TmpItem.iLeft, TmpItem.iTop, TmpItem.iWidth, TmpItem.iHeight,
					hBGCurWnd, (HMENU) iId, hBGInstance, NULL);
		if (hItem == NULL)
		{
			BG_ErrorDisplay (BG_ERR_W32CREATEWINDOW);
			return BG_ERR_W32CREATEWINDOW;
		}

		// save item handle and id
		TmpItem.hHandle = hItem;
		TmpItem.iId = iId;
		BGItem [sBGFormName + "." + sBGItemName] = TmpItem;

		// enable button
		if (!TmpItem.bEnable)
			EnableWindow (hItem, false);

		// restore error message
		BG_ErrorRestore ();

		return 0;
	}


	// create BUTTON item
	if (TmpItem.sType == BG_ITEM_BUTTON)
	{
		// define style of the item
		dwStyle = WS_CHILD | WS_TABSTOP | BS_OWNERDRAW;
		if (!TmpItem.bVisible)
			dwStyle = dwStyle | WS_DISABLED;
		else
			dwStyle = dwStyle | WS_VISIBLE;

		// create button
		hItem = CreateWindowEx (0, "BUTTON", sBGItemName.c_str (), dwStyle,
					TmpItem.iLeft, TmpItem.iTop, TmpItem.iWidth, TmpItem.iHeight,
					hBGCurWnd, (HMENU) iId, hBGInstance, NULL);
		if (hItem == NULL)
		{
			BG_ErrorDisplay (BG_ERR_W32CREATEWINDOW);
			return BG_ERR_W32CREATEWINDOW;
		}

		// save item handle and id
		TmpItem.hHandle = hItem;
		TmpItem.iId = iId;
		BGItem [sBGFormName + "." + sBGItemName] = TmpItem;

		// enable button
		if (!TmpItem.bEnable)
			EnableWindow (hItem, false);

		// restore error message
		BG_ErrorRestore ();

		return 0;
	}


	// create system CHECKBOX item
	if (TmpItem.sType == BG_ITEM_CHECKSYS)
	{
		// define style of the item
		dwStyle = WS_CHILD | WS_TABSTOP | BS_AUTOCHECKBOX | BS_NOTIFY;
		if (!TmpItem.bVisible)
			dwStyle = dwStyle | WS_DISABLED;
		else
			dwStyle = dwStyle | WS_VISIBLE;

		// create button
		hItem = CreateWindowEx (0, "BUTTON", sBGItemName.c_str (), dwStyle,
					TmpItem.iLeft, TmpItem.iTop, 14, 14,
//					TmpItem.iWidth, TmpItem.iHeight,
					hBGCurWnd, (HMENU) iId, hBGInstance, NULL);
		if (hItem == NULL)
		{
			BG_ErrorDisplay (BG_ERR_W32CREATEWINDOW);
			return BG_ERR_W32CREATEWINDOW;
		}

		// save item handle and id
		TmpItem.hHandle = hItem;
		TmpItem.iId = iId;
		BGItem [sBGFormName + "." + sBGItemName] = TmpItem;

		// initialise checkbox
		if (TmpItem.bCheck)
			SendMessage (hItem, BM_SETCHECK, BST_CHECKED, 0);

		// enable checkbox
		if (!TmpItem.bEnable)
			EnableWindow (hItem, false);

		// restore error message
		BG_ErrorRestore ();

		return 0;
	}



	// create system COMBOBOX item
	if (TmpItem.sType == BG_ITEM_COMBOSYS)
	{
		// define style of the item
		dwStyle = WS_TABSTOP | WS_CHILD | CBS_AUTOHSCROLL | WS_VSCROLL;
		if (!TmpItem.bVisible)
			dwStyle = dwStyle | WS_DISABLED;
		else
			dwStyle = dwStyle | WS_VISIBLE;

		if (TmpItem.bEdit)
				dwStyle = dwStyle | CBS_DROPDOWN;
		else
				dwStyle = dwStyle | CBS_DROPDOWNLIST;

		// create Combo item
		hItem = CreateWindowEx (0, "ComboBox", NULL, dwStyle,
					TmpItem.iLeft, TmpItem.iTop, TmpItem.iWidth, TmpItem.iHeight,
					hBGCurWnd, (HMENU) iId, hBGInstance, NULL);
		if (hItem == NULL)
		{
			BG_ErrorDisplay(BG_ERR_W32CREATEWINDOW);
			return BG_ERR_W32CREATEWINDOW;
		}


		// set max number of char
		if (TmpItem.iLength > 0 || TmpItem.iLength < 32766)
			SendMessage (hItem, EM_LIMITTEXT, TmpItem.iLength, 0);

		// save item handle and id
		TmpItem.hHandle = hItem;
		TmpItem.iId = iId;
		BGItem [sBGFormName + "." + sBGItemName] = TmpItem;


		// load values into combo box
		for (iTemp = 1; iTemp < 51; iTemp++)
		{
			// search value into item param
			sString = BG_PRM_VALUE + BG_IntToString (iTemp);
			sString = BG_StringParamGetInt (sString, TmpItem.sParam);
			if (sString == "")
				break;

			// send text to combo box
			SendMessage (hItem, CB_ADDSTRING, 0, (LPARAM) sString.c_str()); 
		}

		//set width of drop list
//		SendMessage (hItem, CB_SETDROPPEDWIDTH, (WPARAM) 100, 0);

		// select first line
		SendMessage (hItem, CB_SETCURSEL, 0, 0);


		// enable combobox
		if (!TmpItem.bEnable)
			EnableWindow (hItem, false);

		// restore error message
		BG_ErrorRestore ();

		return 0;
	}

	// restore error message
	BG_ErrorRestore ();

	return 0;

}



//-----------------------------------------------------------------
//!    BG_WindowSelect : Prepare window for drawing               !
//-----------------------------------------------------------------
//! sBGName : internal file name                                  !
//!																  !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_WindowSelect (const string sBGName)
{
	BG_STRUCT_WINDOW TmpWindow;
	BGWindowItDef BGWindowIt;
	int iError;

	// for error management
	BG_ErrorSet (sBGName, "BG_WindowSelect");

	// search window
	TmpWindow = BG_WindowGet (sBGName);
	if (TmpWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_NOTOPEN);
		return BG_ERR_NOTOPEN;
	}
	
	// prepare window for painting
	hBGdc = BeginPaint (TmpWindow.hBGWnd, &psBGps);
	if (hBGdc == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32BEGINPAINT);
		return BG_ERR_W32BEGINPAINT;
	}

	// memorize selected window
	BGCurWindow = sBGName;

	// clear window
	iError = BG_WindowCls ();
	if (iError != 0)
		return iError;

/*
	// hide scroll bar
	ShowScrollBar (BGWindow [iBGCurWindow].hBGWnd, SB_BOTH, false);
*/

	return 0;

}



//-----------------------------------------------------------------
//!    BG_WindowRelease : release window from drawing             !
//-----------------------------------------------------------------
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_WindowRelease (void)
{

	int iError;
	RECT rcBGwindow;	// size of the window
//	SCROLLINFO BGScrollInfo;
	BG_STRUCT_WINDOW TmpWindow;


	// for error management
	BG_ErrorSet ("", "BG_WindowRelease");

	// search current window
	TmpWindow = BG_WindowGet (BGCurWindow);
	if (TmpWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_NOTOPEN);
		return BG_ERR_NOTOPEN;
	}

	// get the client size of the window
	iError = GetClientRect(TmpWindow.hBGWnd, &rcBGwindow);
	if(iError == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32GETCLIENTRECT);
		return BG_ERR_W32GETCLIENTRECT;
	}

/*
	// display vertical scroll bar, if needed and one time
//	if (BGWindow [iBGCurWindow].iYDrawPos >= rcBGwindow.bottom &&
//		!BGWindow [iBGCurWindow].bVScrollBar)
	if (TmpWindow.bInvisible && !TmpWindow.bVScrollBar)
	{
		iError = ShowScrollBar(TmpWindow.hBGWnd, SB_VERT, true);
		if (iError == 0)
		{
			BG_ErrorDisplay (BG_ERR_W32SHOWSCROLLBAR);
			return BG_ERR_W32SHOWSCROLLBAR;
		}

		// move the scroll thumb
		BGScrollInfo.cbSize = sizeof(SCROLLINFO);
		BGScrollInfo.fMask = SIF_POS  | SIF_RANGE | SIF_PAGE;
		BGScrollInfo.nPos = 0;
		BGScrollInfo.nMin = 0;
		BGScrollInfo.nMax = TmpWindow.iLineCnt - 1;
		BGScrollInfo.nPage = TmpWindow.iLastLineCnt - TmpWindow.iFirstLineCnt + 1;

		// (no error management)
		SetScrollInfo (TmpWindow.hBGWnd, SB_VERT, &BGScrollInfo, TRUE);

		// redraw the window because ShowScrollBar erase it !
		iError = BG_WindowRedraw ();
		if (iError != 0)
			return iError;

		// vertical scroll bar is active
		TmpWindow.bVScrollBar = true;
		BGWindow [BGCurWindow] = TmpWindow;
	}
*/

	// release window
	EndPaint (TmpWindow.hBGWnd, &psBGps);

/*	// reset current window
	iBGCurWindow = 0;   no because WindowMessageAuto
*/

	return 0;

}



//-----------------------------------------------------------------
//!    BG_WindowMessageAuto : auto process window proc message    !
//-----------------------------------------------------------------
//! message : message from messageloop                            !
//! wParam : wParam from messageloop                              !
//!																  !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
//!	process : WM_SIZE, WM_DESTROY, WM_VSCROLL                     !
//-----------------------------------------------------------------
int BG_WindowMessageAuto (const UINT message, const WPARAM wParam)
{

	int iError;
	RECT rcWindow;			// size of the window
	SCROLLINFO BGScrollInfo;
	BG_STRUCT_WINDOW TmpWindow;


	// for error management
	BG_ErrorSet ("", "BG_WindowMessageAuto");

	// get the current window structure
	TmpWindow = BG_WindowGet (BGCurWindow);

	switch (message)
	{
		// user change window size
		case WM_SIZE:
		{
			// get the client size of the window
			iError = GetClientRect (TmpWindow.hBGWnd, &rcWindow);
			if(iError == 0)
			{
				BG_ErrorDisplay (BG_ERR_W32GETCLIENTRECT);
				return BG_ERR_W32GETCLIENTRECT;
			}

			// erase vertical scroll bar
			if (TmpWindow.iYDrawPos < rcWindow.bottom &&
				TmpWindow.iFirstLineCnt == 0)
			{
				iError = ShowScrollBar (TmpWindow.hBGWnd, SB_VERT, false);
				if (iError == 0)
				{
					BG_ErrorDisplay (BG_ERR_W32SHOWSCROLLBAR);
					return BG_ERR_W32SHOWSCROLLBAR;
				}
			}

			break;
		}

		// user use the vertical scroll bar
		case WM_VSCROLL:
		{
			int iBGFirstLineSave;
			int iBGLastLineSave;
			int iBGLinePerPage;


			// save line position
			iBGFirstLineSave = TmpWindow.iFirstLineCnt;
			iBGLastLineSave = TmpWindow.iLastLineCnt;
			
			// number of line in the page
			iBGLinePerPage = iBGLastLineSave - iBGFirstLineSave;

			switch (LOWORD (wParam))
			{
				// user move the thumb
				case SB_THUMBTRACK:
				{
					TmpWindow.iFirstLineCnt = HIWORD (wParam);
					TmpWindow.iLastLineCnt = HIWORD (wParam) + iBGLinePerPage;
					break;
				}

				// 1 line down
				case SB_LINEDOWN:
				{
					TmpWindow.iFirstLineCnt += 1;
					TmpWindow.iLastLineCnt += 1;
					break;
				}

				// 1 line up
				case SB_LINEUP:
				{
					TmpWindow.iFirstLineCnt -= 1;
					TmpWindow.iLastLineCnt -= 1;
					break;
				}

				// 1 page up
				case SB_PAGEUP:
				{
					TmpWindow.iFirstLineCnt -= iBGLinePerPage;
					TmpWindow.iLastLineCnt -= iBGLinePerPage;
					break;
				}

				// 1 page down
				case SB_PAGEDOWN:
				{
					TmpWindow.iFirstLineCnt += iBGLinePerPage;
					TmpWindow.iLastLineCnt += iBGLinePerPage;
					break;
				}

			}

			// compute again, number of line in the page
			iBGLinePerPage = TmpWindow.iLastLineCnt - TmpWindow.iFirstLineCnt;

			// limit to first line and last line
			if (TmpWindow.iFirstLineCnt < 0)
			{
				TmpWindow.iFirstLineCnt = 0;
				TmpWindow.iLastLineCnt = iBGLastLineSave;
			}

//			if (!BGWindow [iBGCurWindow].bInvisible &&
			if (TmpWindow.iFirstLineCnt >= TmpWindow.iLineCnt - iBGLinePerPage)
			{
				TmpWindow.iLastLineCnt = TmpWindow.iLineCnt - 1;
				TmpWindow.iFirstLineCnt = TmpWindow.iLastLineCnt - iBGLinePerPage;
			}

			// save window structure info
			BGWindow [BGCurWindow] = TmpWindow;

			// redraw window
			if (TmpWindow.iFirstLineCnt != iBGFirstLineSave)
			{
				// move the scroll thumb
				BGScrollInfo.cbSize = sizeof(SCROLLINFO);
				BGScrollInfo.fMask = SIF_POS  | SIF_RANGE | SIF_PAGE;
				BGScrollInfo.nPos = TmpWindow.iFirstLineCnt;
				BGScrollInfo.nMin = 0;
				BGScrollInfo.nMax = TmpWindow.iLineCnt - 1;
				BGScrollInfo.nPage = TmpWindow.iLastLineCnt - TmpWindow.iFirstLineCnt + 1;

				// (no error management)
				SetScrollInfo (TmpWindow.hBGWnd, SB_VERT, &BGScrollInfo, TRUE);

				// redraw the window
				iError = BG_WindowRedraw ();
				if (iError != 0)
					return iError;

			}

			break;

		}

		// user close the window
		case WM_DESTROY:
			PostQuitMessage(0);

		break;
	}

	return 0;

}



//-----------------------------------------------------------------
//!    BG_WindowCls : Clear current window                        !
//-----------------------------------------------------------------
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
//! call by BG_WindowSelect                                       !
//-----------------------------------------------------------------
int BG_WindowCls (void)
{

	BG_STRUCT_WINDOW TmpWindow;
	int iError;


	// for error management
	BG_ErrorSet ("", "BG_WindowCls");

	// get the current window structure
	TmpWindow = BG_WindowGet (BGCurWindow);
	if (TmpWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_NOTOPEN);
		return BG_ERR_NOTOPEN;
	}

	// reset draw position
	TmpWindow.iXDrawPos = 0;
	TmpWindow.iYDrawPos = 0;
	TmpWindow.iLineCnt = 0;

	// reset font to default
	TmpWindow.sFontName = sBG_DefFontName;
	TmpWindow.iFontHeight = iBG_DefFontHeight;
	iError = BG_TextFontSet (sBG_DefFontName, iBG_DefFontHeight, 0, 0, 0);
	if (iError != 0)
		return iError;

	TmpWindow.bFontBold = false;
	TmpWindow.bFontItalic = false;
	TmpWindow.bUnderline = false;

	// save window structure info
	BGWindow [BGCurWindow] = TmpWindow;

	return 0;

}



//-----------------------------------------------------------------
//!    BG_TextDraw : Display text on screen                       !
//-----------------------------------------------------------------
//! sBGText : text to print                                       !
//! sBGParam : parameters (default = "")                          !
//!																  !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_TextDraw (const string sBGText, const string sBGParam)
				// sParam = ""
{

	SIZE szSize;
	RECT rcWindow;	// size of the window
	BG_STRUCT_WINDOW TmpWindow;
	string sString;
	bool bBGlf;
	int iError;


	// for error management
	BG_ErrorSet ("", "BG_TextDraw");

	// get the current window structure
	TmpWindow = BG_WindowGet (BGCurWindow);
	if (TmpWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_NOTOPEN);
		return BG_ERR_NOTOPEN;
	}

	// default is NOT LineFeed
	bBGlf = false;

	// search border parameter
	sString = BG_StringParamGetInt (BG_PRM_LF, sBGParam);
	if (BG_StringIsTrue (sString))
		bBGlf = true;

	// get the client size of the window
	iError = GetClientRect (TmpWindow.hBGWnd, &rcWindow);
	if(iError == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32GETCLIENTRECT);
		return BG_ERR_W32GETCLIENTRECT;
	}

	// don't draw text if it is not visible
	if (TmpWindow.iYDrawPos < rcWindow.bottom &&
	   TmpWindow.iLineCnt >= TmpWindow.iFirstLineCnt)
	{
		// draw text if it is empty
		if (sBGText != "")
		{
			// select back ground color
			iError = SetBkColor (hBGdc, TmpWindow.iBackColor);
			if (iError == CLR_INVALID)
			{
				BG_ErrorDisplay (BG_ERR_W32SETBKCOLOR );
				return BG_ERR_W32SETBKCOLOR ;
			}
/*
//SetBkColor(hBGdc,BG_COL_LIGHTGREY);
//SetBkMode(hBGdc, TRANSPARENT);

//########################################################################"
if (BGWindow [iBGCurWindow].bDrawBackColor)
{
GetTextExtentPoint32(hBGdc, sBGText.data(), sBGText.length(), &BGsz); 
RECT xrect;
HBRUSH hbrush;

xrect.left = 0;
xrect.top = BGWindow [iBGCurWindow].iYDrawPos;
xrect.right = 2000; //###a modifier
xrect.bottom = BGWindow [iBGCurWindow].iYDrawPos+BGsz.cx;
if ((BGWindow [iBGCurWindow].iLineCnt / 2.0) == (BGWindow [iBGCurWindow].iLineCnt / 2))
{
	hbrush = CreateSolidBrush(RGB(200,255,200)); // lightgreen
}
else
{
	hbrush = CreateSolidBrush(RGB(255,255,200)); // lightyellow
}
FillRect(hBGdc,&xrect,hbrush);
SetBkMode(hBGdc, TRANSPARENT);
BGWindow [iBGCurWindow].bDrawBackColor = false;
DeleteObject(hbrush);
}
//####################################################################
*/
			// draw text on window
			iError = TextOut (hBGdc, TmpWindow.iXDrawPos, TmpWindow.iYDrawPos,
							sBGText.data(), sBGText.length());
			if (iError == 0)
			{
				BG_ErrorDisplay (BG_ERR_W32TEXTOUT);
				return BG_ERR_W32TEXTOUT;
			}			

			// get the size of the text
			iError = GetTextExtentPoint32 (hBGdc, sBGText.data(), sBGText.length(), &szSize); 
			if (iError == 0)
			{
				BG_ErrorDisplay (BG_ERR_W32GETTEXTEXPT32);
				return BG_ERR_W32GETTEXTEXPT32;
			}
		}
		// we only want a line feed
		else
		{
			iError = GetTextExtentPoint32 (hBGdc, " ", 1, &szSize);
			if (iError == 0)
			{
				BG_ErrorDisplay (BG_ERR_W32GETTEXTEXPT32);
				return BG_ERR_W32GETTEXTEXPT32;
			}
		}

		// move the next draw position
		if (bBGlf)
		{
			TmpWindow.iXDrawPos = 0;
			TmpWindow.iYDrawPos += szSize.cy;
			TmpWindow.iLastLineCnt = TmpWindow.iLineCnt;
			TmpWindow.iLineCnt += 1;
			TmpWindow.bInvisible = false;
		}
		else
		{
			TmpWindow.iXDrawPos += szSize.cx;
		}

	}
	// text is invisible, only increment line counter
	else
	{
		if (bBGlf)
		{
			TmpWindow.iXDrawPos = 0;
			TmpWindow.iLineCnt += 1;
			TmpWindow.bInvisible = true;
//			TmpWindow.iMaxLineCpt += 1;
		}

	}

	// save window structure info
	BGWindow [BGCurWindow] = TmpWindow;


	return 0;

}



//-----------------------------------------------------------------
//!    BG_TextPositionSet : Change drawing position               !
//-----------------------------------------------------------------
//! iBGHPos : horizontal position (in pixel)                      !
//!																  !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_TextPositionSet (const int iBGHPos)
{
	BG_STRUCT_WINDOW TmpWindow;

	// for error management
	BG_ErrorSet ("", "BG_TextPositionSet");

	// get the current window structure
	TmpWindow = BG_WindowGet (BGCurWindow);
	if (TmpWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_NOTOPEN);
		return BG_ERR_NOTOPEN;
	}

	if (iBGHPos < 0)
	{
		BG_ErrorDisplay (BG_ERR_NOTINRANGE);
		return BG_ERR_NOTINRANGE;
	}

	// move cursor to draw position
	if (iBGHPos != -1)
		TmpWindow.iXDrawPos = iBGHPos;

	// save window structure info
	BGWindow [BGCurWindow] = TmpWindow;


	return 0;

}



//-----------------------------------------------------------------
//!    BG_TextColorSet : Change current text color                !
//-----------------------------------------------------------------
//! sBGColor : define color or hexa value                         !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_TextColorSet (const string sBGColor)
{
	COLORREF cNewColor;
	COLORREF cOldColor;


	// for error management
	BG_ErrorSet ("", "BG_TextColorSet");

	// select color
	cNewColor = BG_ColorNumGet (sBGColor);

	// change the text color
	cOldColor = SetTextColor (hBGdc, cNewColor);
	if (cOldColor == CLR_INVALID)
	{
		BG_ErrorDisplay (BG_ERR_W32SETTEXTCOLOR);
		return BG_ERR_W32SETTEXTCOLOR;
	}
	

	return 0;

}



//-----------------------------------------------------------------
//!    BG_TextFontSet : Change current font                       !
//-----------------------------------------------------------------
//! sBGFontName : name of the font to use                         !
//!				   if == "" : keep last font                      !
//! iBGFontHeight : height of the font                            !
//! iBGBold : bold           \   ( 0 = no                         !
//! iBGItalic : italic        =- ( 1 = yes                        !
//! iBGUnderline: underline  /   ( other = don't change           !
//!                              ( default = don't change         !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_TextFontSet (const string sBGFontName, const int iBGFontHeight,
				const int iBGBold, const int iBGItalic, const int iBGUnderline)
				// iBGBold = -1, iBGItalic = -1, iBGUnderline = -1
{
	int iTmpWeight;
	HFONT hfont;
	int iHFont;
	int iError;
	HGDIOBJ hError;
	BG_STRUCT_WINDOW TmpWindow;


	// for error management
	BG_ErrorSet ("", "BG_TextFontSet");

	// get the current window structure
	TmpWindow = BG_WindowGet (BGCurWindow);
	if (TmpWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_NOTOPEN);
		return BG_ERR_NOTOPEN;
	}

	// modify BG bold,italic,underline values
	if (iBGBold == 0)
	{
		TmpWindow.bFontBold = false;
		iTmpWeight = FW_REGULAR;
	}
	else if (iBGBold == 1)
	{
		TmpWindow.bFontBold = true;
		iTmpWeight = FW_BOLD;
	}
	else
	{
		if (TmpWindow.bFontBold)
			iTmpWeight = FW_BOLD;
		else
			iTmpWeight = FW_REGULAR;
	}

	if (iBGItalic == 0)
		TmpWindow.bFontItalic = false;
	else if (iBGItalic == 1)
		TmpWindow.bFontItalic = true;

	if (iBGUnderline == 0)
		TmpWindow.bUnderline = false;
	else if (iBGUnderline == 1)
		TmpWindow.bUnderline = true;


	// modify BG current font 
	if (sBGFontName != "")
		TmpWindow.sFontName = sBGFontName;

	// modify BG current font height
	if (iBGFontHeight != 0)
		TmpWindow.iFontHeight = iBGFontHeight;

	// compute font size
	iHFont = -MulDiv (TmpWindow.iFontHeight, GetDeviceCaps (hBGdc, LOGPIXELSY), 72);


	// save window structure info
	BGWindow [BGCurWindow] = TmpWindow;

	// set the new font
	hfont = CreateFont(
				iHFont,								// height of font
				0,									// average character width
				0,									// angle of escapement
				0,									// base-line orientation angle
				iTmpWeight,							// font weight
				TmpWindow.bFontItalic,				// italic attribute option
				TmpWindow.bUnderline,				// underline attribute option
				false,								// strikeout attribute option
				ANSI_CHARSET,						// character set identifier
				OUT_DEFAULT_PRECIS,					// output precision
				CLIP_DEFAULT_PRECIS,				// clipping precision
				DEFAULT_QUALITY,					// output quality
				DEFAULT_PITCH | FF_SWISS,			// pitch and family
				TmpWindow.sFontName.data());		// typeface name

	if (hfont == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32CREATEFONT);
		return BG_ERR_W32CREATEFONT;
	}

	// set and delete font object
	hError = SelectObject (hBGdc, hfont);
	if (hError == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32DELETEOBJECT);
		return BG_ERR_W32DELETEOBJECT;
	}

	iError = DeleteObject (hfont);
	if (iError == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32DELETEOBJECT);
		return BG_ERR_W32DELETEOBJECT;
	}

	return 0;

}



//-----------------------------------------------------------------
//!    BG_InputBox : Display an input box and wait answer         !
//-----------------------------------------------------------------
//! sBGText : title of input field                                !
//! sBGTitle : title of the input box, 100 char max               !
//!              if == "" then = "BipGDI InputBox"                !
//! sBGDefValue : default value for input field                   !
//!                (default value = "")                           !
//!                                                               !
//! return : string (max 255 char)                                !
//!          or "" and BGInputBox if error                        !
//-----------------------------------------------------------------
string BG_InputBox (const string sBGText, const string sBGTitle, const string sBGDefValue)
					// sBGTitle = "", sBGDefValue = ""
{
	HGLOBAL hgMem;
	LPDLGTEMPLATE lpDlgtplt;
	LPWORD lpWord;
	LPWSTR lpWstr;
	string sDlgFont;
	string sTmpTitle;
	int iError;
	int iNbChar;
	BG_STRUCT_WINDOW MainWindow;


	// for error management
	BG_ErrorSet (sBGTitle, "BG_InputBox");

	//initialise inputbox structure
	BGInputBox.sMessage = sBGText;
	BGInputBox.sReturnString = sBGDefValue;
	BGInputBox.bOk = false;
	BGInputBox.bCancel = false;
	BGInputBox.iReturnCode = 0;


	// main window not open
	MainWindow = BG_WindowGet (BGMainWindow);
	if (MainWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_MAINNOTOPEN);
		BGInputBox.sReturnString = "";
		BGInputBox.bOk = false;
		BGInputBox.bCancel = false;
		BGInputBox.iReturnCode = BG_ERR_MAINNOTOPEN;
		return "";
	}


	// assign default value for the inputbox title
	if (sBGTitle == "")
		sTmpTitle = "BipGDI InputBox";

	// limit the title to 100 char
	if (sBGTitle.length() > 100)
		sTmpTitle = sBGTitle.substr (0, 100);


	// *** create dynamic dialog box ***

	// allocate memory for inputbox template
	hgMem = GlobalAlloc (GMEM_ZEROINIT, 1024);
	if (hgMem == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32GLOBALALLOC);
		BGInputBox.sReturnString = "";
		BGInputBox.bOk = false;
		BGInputBox.bCancel = false;
		BGInputBox.iReturnCode = BG_ERR_W32GLOBALALLOC;
		return "";
	}

	// lock template memory
	lpDlgtplt = (LPDLGTEMPLATE) GlobalLock (hgMem);
	if (lpDlgtplt == NULL)
	{
		// free memory, no error verify
		GlobalFree (hgMem);

		BG_ErrorDisplay (BG_ERR_W32GLOBALLOCK);
		BGInputBox.sReturnString = "";
		BGInputBox.bOk = false;
		BGInputBox.bCancel = false;
		BGInputBox.iReturnCode = BG_ERR_W32GLOBALLOCK;
		return "";
	}


	// description of dialog box
	lpDlgtplt->style = WS_POPUP | WS_SYSMENU | WS_CAPTION | DS_MODALFRAME |
						 DS_CENTER | WS_VISIBLE | DS_SETFONT;
	lpDlgtplt->dwExtendedStyle = 0;
	lpDlgtplt->cdit = 0;		// controls are create in BGPopUpProc
	lpDlgtplt->x = 50;
	lpDlgtplt->y = 50;
	lpDlgtplt->cx = 200;
	lpDlgtplt->cy = 70;

	lpWord = (LPWORD) (lpDlgtplt + 1);
	*lpWord++ = 0;   // no menu
	*lpWord++ = 0;   // predefined dialog box class

	// dialog box title
	lpWstr = (LPWSTR) lpWord;
	iNbChar = 1 + MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, sTmpTitle.c_str(),
						 sTmpTitle.length(), lpWstr, sTmpTitle.length());
	lpWord += iNbChar;

	// dialog box font
	*lpWord++ = 10;   // font size

	sDlgFont = "Arial";
	lpWstr = (LPWSTR) lpWord;
	iNbChar = 1 + MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, sDlgFont.c_str(),
						 sDlgFont.length(), lpWstr, sDlgFont.length());
	lpWord += iNbChar;


	// unlock template memory
	iError = GlobalUnlock (hgMem);
	if (iError != 0)
//	modify because of Win98
//	if (iError == 0 && GetLastError() != NO_ERROR)
	{
		// free memory, no error verify
		GlobalFree (hgMem);

		BG_ErrorDisplay (BG_ERR_W32GLOBALUNLOCK);
		BGInputBox.sReturnString = "";
		BGInputBox.bOk = false;
		BGInputBox.bCancel = false;
		BGInputBox.iReturnCode = BG_ERR_W32GLOBALUNLOCK;
		return "";
	}


	// create dialogbox
	iError = DialogBoxIndirect (hBGInstance, (LPDLGTEMPLATE) hgMem,
                             MainWindow.hBGWnd, (DLGPROC) BG_InputBoxProcIntl);
	if (iError == -1)
	{
		// free memory, no error verify
		GlobalFree (hgMem);
		
		BG_ErrorDisplay (BG_ERR_W32DIALOGBOXIND);
		BGInputBox.sReturnString = "";
		BGInputBox.bOk = false;
		BGInputBox.bCancel = false;
		BGInputBox.iReturnCode = BG_ERR_W32DIALOGBOXIND;
		return "";
	}

	// free inputbox template
	hgMem = GlobalFree (hgMem);
	if (hgMem != NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32GLOBALFREE);
		BGInputBox.sReturnString = "";
		BGInputBox.bOk = false;
		BGInputBox.bCancel = false;
		BGInputBox.iReturnCode = BG_ERR_W32GLOBALFREE;
		return "";
	}


	return BGInputBox.sReturnString;


}



//-----------------------------------------------------------------
//!    BG_InputBoxErrorGet : Give last return code from InputBox  !
//-----------------------------------------------------------------
//! return : error                                                !
//-----------------------------------------------------------------
int BG_InputBoxErrorGet (void)
{

	return BGInputBox.iReturnCode;

}



//-----------------------------------------------------------------
//!    BG_InputBoxProcIntl : Window Procedure for BG_InputBox     !
//-----------------------------------------------------------------
//! hBGCurWnd : current window handle                             !
//! message : message from message loop                           !
//! wParam : wParam from message loop                             !
//! lParam : lParam from message loop                             !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
//! INTERNAL FUNCTION : use by BG_InputBox                        !
//-----------------------------------------------------------------
int CALLBACK BG_InputBoxProcIntl (HWND hBGCurWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	const int BG_OK = 100;
	const int BG_EDIT = 101;

	char szText [255];
	int iError;
	HWND hError;


	// for error management
	BG_ErrorSet ("", "BG_InputBoxProcIntl");


	switch (message)
	{
		case WM_CLOSE:
		{
			BGInputBox.bCancel = true;
			BGInputBox.sReturnString = "";

			EndDialog (hBGCurWnd, 0);
			return true;
		}

		case WM_INITDIALOG:
		{

			HWND hStatic;			// handle for static text
			HWND hEdit;				// handle for edit box
			HWND hOk;				// handle for OK button

			// create static text
			hStatic = CreateWindowEx (0, "STATIC", BGInputBox.sMessage.c_str(),
						WS_VISIBLE | WS_CHILD, 20, 20, 220, 20, hBGCurWnd, NULL,
						hBGInstance, NULL);

			if (hStatic == NULL)
			{
				BG_ErrorDisplay (BG_ERR_W32CREATEWINDOW);
				return BG_ERR_W32CREATEWINDOW;
			}

			// set static field text
			iError = SetWindowText (hStatic, BGInputBox.sMessage.c_str());

			if (iError == 0)
			{
				BG_ErrorDisplay (BG_ERR_W32SETWINDOWTEXT);
				return BG_ERR_W32SETWINDOWTEXT;
			}

			// create edit box
			hEdit = CreateWindowEx (WS_EX_OVERLAPPEDWINDOW, "EDIT", "",
						WS_VISIBLE | WS_CHILD | WS_TABSTOP | ES_AUTOHSCROLL,
						20, 40, 300, 20, hBGCurWnd, (HMENU) BG_EDIT, hBGInstance, NULL);

			if (hEdit == NULL)
			{
				BG_ErrorDisplay (BG_ERR_W32CREATEWINDOW);
				return BG_ERR_W32CREATEWINDOW;
			}

			// set edit box default text
			iError = SetWindowText (hEdit, BGInputBox.sReturnString.c_str());

			if (iError == 0)
			{
				BG_ErrorDisplay (BG_ERR_W32SETWINDOWTEXT);
				return BG_ERR_W32SETWINDOWTEXT;
			}

			// select all the text
			SendMessage (hEdit, EM_SETSEL, 0, -1);

			// set focus to edit box
			hError = SetFocus (hEdit);
//			return handle is always NULL ???
//			if (hError == NULL)
//			{
//				BG_ErrorDisplay (BG_ERR_W32SETFOCUS);
//				return BG_ERR_W32SETFOCUS;
//			}

			// create OK Button
			hOk = CreateWindowEx (0, "BUTTON", "OK",
						WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON | WS_TABSTOP,
						100, 80, 140, 40, hBGCurWnd, (HMENU) BG_OK, hBGInstance, NULL);

			if (hEdit == NULL)
			{
				BG_ErrorDisplay (BG_ERR_W32CREATEWINDOW);
				return BG_ERR_W32CREATEWINDOW;
			}

			return true;
		}

		case WM_COMMAND:
		{
			// OK button is clic or ENTER key is press
			if (LOWORD(wParam) == BG_OK || wParam == IDOK)
			{
				// OK button is pressed
				BGInputBox.bOk = true;

				// get input text
				iError = GetDlgItemText (hBGCurWnd, BG_EDIT, szText, 255);
				if (iError == 0 && szText [0] != 0)
				{
					BG_ErrorDisplay (BG_ERR_W32GETDLTITEMTEXT);
					return BG_ERR_W32GETDLTITEMTEXT;
				}

				// memorize the input text
				BGInputBox.sReturnString = szText;

				// end the dialog box
				iError = EndDialog (hBGCurWnd, 0);
				if (iError == 0)
				{
					BG_ErrorDisplay (BG_ERR_W32ENDDIALOG);
					return BG_ERR_W32ENDDIALOG;
				}

				return true;
			}
		}

//		return 0;
	}

//	return DefWindowProc(hBGCurWnd, message, wParam, lParam);
	return false;

}



//-----------------------------------------------------------------
//!    BG_MessageBox : Display a message box                      !
//-----------------------------------------------------------------
//! sBGTitle : title of the message box                           !
//! sBGText : text to display                                     !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_MessageBox (const string sBGTitle, const string sBGText)
{
	BG_STRUCT_WINDOW MainWindow;


	// for error management
	BG_ErrorSet (sBGTitle, "BG_MessageBox");

	// main window not open
	MainWindow = BG_WindowGet (BGMainWindow);
	if (MainWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_MAINNOTOPEN);
		return BG_ERR_MAINNOTOPEN;
	}

	// display message box
	MessageBox (MainWindow.hBGWnd, sBGText.data(), sBGTitle.data(), MB_OK);

	return 0;

}




//-----------------------------------------------------------------
//!    BG_ErrorDisplaySet : modify the error display flag         !
//-----------------------------------------------------------------
//! bBGError : true  = display error message                      !
//!          false = don't display error message                  !
//! default value is true                                         !
//-----------------------------------------------------------------
void BG_ErrorDisplaySet (const bool bBGError) // bBGError = true
{

	if (bBGError)
		BG_DISPLAY_ERROR = true;
	else
		BG_DISPLAY_ERROR = false;

}



//-----------------------------------------------------------------
//!    BG_ErrorSet : store error message                          !
//-----------------------------------------------------------------
//! sBGName : name of the BipGDI object                           !
//! sBGFunction : name of the BipGDI function                     !
//!																  !
//! return : none                                                 !
//-----------------------------------------------------------------
void BG_ErrorSet (const string sBGName, const string sBGFunction)
{

	BGError.sName = sBGName;
	BGError.sFunction = sBGFunction;
}



//-----------------------------------------------------------------
//!    BG_ErrorSave : save error message into save-structure      !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
void BG_ErrorSave (void)
{

	BGErrorSave.sName = BGError.sName;
	BGErrorSave.sFunction = BGError.sFunction;
}



//-----------------------------------------------------------------
//!    BG_ErrorRestore : restore error message from save-structure!
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
void BG_ErrorRestore (void)
{

	BGError.sName = BGErrorSave.sName;
	BGError.sFunction = BGErrorSave.sFunction;
}



//-----------------------------------------------------------------
//!    BG_ErrorDisplay : display error when using BipGDI          !
//-----------------------------------------------------------------
//! iBGError : error number                                       !
//!																  !
//! return : error message                                        !
//-----------------------------------------------------------------
//!	display error only if flag BG_DISPLAY_ERROR is true           !
//-----------------------------------------------------------------
string BG_ErrorDisplay (const int iBGError)
{

	int iReturn;
	string sMsg;
	string sWinMsg;
	string sWindowName;
	string sString;
	ifstream ErrorStream;
	ofstream OutStream;
	string sErrorMsg;
	string sMsgType;
	string sMsgCat;
	string sMsgLang;
	int iMsgNo;
	string sMsgMsg;
	string sLanguage;
	char cChar20 [20];
	BG_STRUCT_WINDOW TmpWindow;
	LPVOID lpMsgBuf;
	DWORD dwError;


	// if win32 error : get system error number and text
	if (iBGError >= 1000 && iBGError < 2000)
	{
		dwError = GetLastError();
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
		    NULL, dwError,
		    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			(LPTSTR) &lpMsgBuf, 0, NULL);

			sWinMsg = "System error : " + BG_IntToString(dwError) + '\n' +
				"System message : " + (LPCTSTR) lpMsgBuf + "\n";
	}


	// get window structure
	if (BGError.sName != "")
		sWindowName = BGError.sName;
	else
		sWindowName = BGCurWindow;


	// open file containing error message
	ErrorStream.open ("bipmsg.bdb", ios::in);
	if (ErrorStream.fail())
		sErrorMsg = "no message : file bipmsg.bdb not found";
	else
	{
		// select language
		sLanguage = BG_Language;

		// first search in default language. if not found, search in "US " language
		do
		{
			// select US language if default language not found
			if (sLanguage != "US" && ErrorStream.eof())
			{
				sLanguage = "US";

				// clear error state
				ErrorStream.clear();

				// return to the beginning of the file
				ErrorStream.seekg (0, std::ios::beg);
			}

			// search for error message
			do
			{
				// search for begin of record
				do
					getline (ErrorStream, sString);
				while (sString != "<BDBRC>" && !ErrorStream.eof());

				// stop if end of file
				if (ErrorStream.eof())
					break;

				// read next line (supposed to be type)
				getline (ErrorStream, sString);
				if (sString.length() > 2)
					sMsgType = sString.substr (1, sString.length() - 2);

				// read next line (supposed to be cat)
				getline (ErrorStream, sString);
				if (sString.length() > 2)
					sMsgCat = sString.substr (1, sString.length() - 2);

				// read next line (supposed to be lang)
				getline (ErrorStream, sString);
				if (sString.length() > 2)
					sMsgLang = sString.substr (1, sString.length() - 2);

				// read next line (supposed to be no)
				getline (ErrorStream, sString);
				if (sString.length() > 2)
					iMsgNo = BG_StringToInt (sString.substr (1, sString.length() - 2));

				// read next line (supposed to be msg)
				getline (ErrorStream, sString);
				if (sString.length() > 2)
					sMsgMsg = sString.substr (1, sString.length() - 2);

			}
			while (sMsgType != "ERROR" || sMsgCat != "BIPGDI" || sMsgLang != sLanguage
					|| iMsgNo != iBGError || ErrorStream.eof());


		}
		while ((sMsgType != "ERROR" || sMsgCat != "BIPGDI" || iMsgNo != iBGError)
			&& sLanguage != "US");

		if (ErrorStream.eof())
			sErrorMsg = "no message : message error not found";
		else
			sErrorMsg = sMsgMsg;

		// close message error file
		ErrorStream.close ();
	
	}


	// contruct the message to display
	sMsg = "Function : " + BGError.sFunction + '\n' +
		   "Window name : " + sWindowName + '\n' +
		   "Error number : " + BG_IntToString (iBGError) + '\n' +
		   "Error msg : " + sErrorMsg + '\n';


	// if win32 : display windows error message
	if (iBGError >= 1000 && iBGError < 2000)
		sMsg += sWinMsg;


	// Free the buffer.
	LocalFree (lpMsgBuf);
	
	
	// display error if flag is set
	if (BG_DISPLAY_ERROR)
	{
		// contruct the message to display
		sMsg += '\n';
		sMsg += "Do you want to stop the program ?";
	
		// display a messagebox with error icon and 2 button (yes/no)
		iReturn = MessageBox (NULL, sMsg.data(), "BipGDI Error", MB_YESNO | MB_ICONERROR);

		// stop if user clic on "Yes" button, else continue program
		if (iReturn == IDYES)
		{
			// end the program
			PostQuitMessage (0);
		}
	}
	else
	{
		// open output file
		OutStream.open ("biperror.txt", ios::app);
		if (!OutStream.fail())
		{
			// write message to file
			_strdate (cChar20);
			OutStream << cChar20 << ' ';
			_strtime (cChar20);
			OutStream << cChar20 << endl;
			OutStream << sMsg << endl;
			OutStream.close();
		}
	}

	return sMsg;

}



//-----------------------------------------------------------------
//!    BG_WindowTitleSet : Set the title of the window            !
//-----------------------------------------------------------------
//! sBGName : name of the window                                  !
//! sBGWndTitle : string containing the window's title            !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_WindowTitleSet (const string sBGName, const string sBGWndTitle)
{
	BG_STRUCT_WINDOW TmpWindow;
	int iError;


	// for error management
	BG_ErrorSave ();
	BG_ErrorSet (sBGName, "BG_WindowTitleSet");

	// verify window's name
	if (sBGName == "")
		return BG_ERR_BADWINDNAME;

	// get window structure
	TmpWindow = BG_WindowGet (sBGName);
	if (TmpWindow.hBGWnd == NULL)
		return BG_ERR_NOTOPEN;

	// change the title of the window
	iError = SetWindowText (TmpWindow.hBGWnd, sBGWndTitle.data());
	if (iError == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32SETWINDOWTEXT);
		return BG_ERR_W32SETWINDOWTEXT;
	}

	// save window structure info
	TmpWindow.sTitle = sBGWndTitle;
	BGWindow [sBGName] = TmpWindow;

	// restore error message
	BG_ErrorRestore ();

	return 0;

}



//-----------------------------------------------------------------
//!    BG_WindowMove : Define the size of a window and            !
//!                    make it visible                            !
//-----------------------------------------------------------------
//! sBGName : name of the window                                  !
//! iLeft : left position of the window (in pixel)                !
//! iTop : top position of the window (in pixel)                  !
//! iWidth : width of the window (in pixel)                       !
//! iHeight : heigth of the window (in pixel)                     !
//!																  !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
//!	-iLeft is set to 0, if it is < 1 or > screen width            !
//!	-iTop is set to 0, if it is < 1 or > screen height            !
//!	-iWidth is set to screen width, if it is < 1 or > screen width!
//!	-iHeight is set to screen height, if it is < 1 or             !
//!     > screen size height                                      !
//-----------------------------------------------------------------
int BG_WindowMove (const string sBGName, const int iBGLeft, const int iBGTop,
				   const int iBGWidth, const int iBGHeight)
{
	int iLeftDef;
	int iTopDef;
	int iWidthDef;
	int iHeightDef;
	RECT rcScreen;
	BG_STRUCT_WINDOW TmpWindow;
	BG_STRUCT_WINDOW MainWindow;
	int iError;


	// for error management
	BG_ErrorSave ();
	BG_ErrorSet (sBGName, "BG_WindowMove");

	// get window structure
	TmpWindow = BG_WindowGet (sBGName);
	if (TmpWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_NOTOPEN);
		return BG_ERR_NOTOPEN;
	}

	// if main window
	if (sBGName == BGMainWindow)
	{
		// get the size of the screen
		iError = SystemParametersInfo (SPI_GETWORKAREA, 0, &rcScreen, 0);
		if (iError == 0)
		{
			BG_ErrorDisplay (BG_ERR_W32SYSPARAMINFO);
			return BG_ERR_W32SYSPARAMINFO;
		}

		// default size if full screen
		if (iBGLeft < 0 && iBGLeft > rcScreen.right)
			iLeftDef = rcScreen.left;
		else
			iLeftDef = iBGLeft;

		// default size if full screen		
		if (iBGTop < 0 && iBGTop > rcScreen.top)
			iTopDef = rcScreen.top;
		else
			iTopDef = iBGTop;

		// default size if full screen		
		if (iBGWidth < 1 || iBGWidth > rcScreen.right)
			iWidthDef = rcScreen.right;
		else
			iWidthDef = iBGWidth;

		// default size if full screen		
		if (iBGHeight < 1 || iBGHeight > rcScreen.bottom)
			iHeightDef = rcScreen.bottom;
		else
			iHeightDef = iBGHeight;

	}
	else
	{
		// get MAIN window structure
		MainWindow = BG_WindowGet (BGMainWindow);
		if (MainWindow.hBGWnd == NULL)
		{
			BG_ErrorDisplay (BG_ERR_NOTOPEN);
			return BG_ERR_NOTOPEN;
		}

		// get the client size of the main window
		iError = GetClientRect (MainWindow.hBGWnd, &rcScreen);
		if(iError == 0)
		{
			BG_ErrorDisplay (BG_ERR_W32GETCLIENTRECT);
			return BG_ERR_W32GETCLIENTRECT;
		}


		// default size if full screen		
		if (iBGLeft < 0 || iBGLeft >= rcScreen.right)
			// don't change
			iLeftDef = TmpWindow.iLeft;
		else
			iLeftDef = iBGLeft;

		// default size if full screen		
		if (iBGTop < 0 || iBGTop >= rcScreen.bottom)
			// don't change
			iTopDef = TmpWindow.iTop;
		else
			iTopDef = iBGTop;

		// default size if full screen		
		if (iBGWidth < 1 || iBGWidth > rcScreen.right)
			iWidthDef = rcScreen.right;
		else
			iWidthDef = iBGWidth;

		// default size if full screen		
		if (iBGHeight < 1 || iBGHeight > rcScreen.bottom)
			iHeightDef = rcScreen.bottom;
		else
			iHeightDef = iBGHeight;

	}

	// save window structure info
	TmpWindow.iLeft = iLeftDef;
	TmpWindow.iTop = iTopDef;
	TmpWindow.iWidth = iWidthDef;
	TmpWindow.iHeight = iHeightDef;
	BGWindow [sBGName] = TmpWindow;


	iError = MoveWindow (TmpWindow.hBGWnd, TmpWindow.iLeft,
		TmpWindow.iTop, TmpWindow.iWidth, TmpWindow.iHeight, TRUE);
	if (iError == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32MOVEWINDOW);
		return BG_ERR_W32MOVEWINDOW;
	}

	// display window
	ShowWindow (TmpWindow.hBGWnd, SW_SHOWNORMAL);

	// restore error message
	BG_ErrorRestore ();

	return 0;

}



//-----------------------------------------------------------------
//!    BG_WindowRedraw : redraw current window                    !
//-----------------------------------------------------------------
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_WindowRedraw (void)
{
	BG_STRUCT_WINDOW TmpWindow;
	int iError;


	// get the current window structure
	TmpWindow = BG_WindowGet (BGCurWindow);
	if (TmpWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_NOTOPEN);
		return BG_ERR_NOTOPEN;
	}

	iError = InvalidateRect (TmpWindow.hBGWnd,NULL, true);
	if (iError == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32INVALIDATERECT);
		return BG_ERR_W32INVALIDATERECT;
	}

	iError = UpdateWindow (TmpWindow.hBGWnd);
	if (iError == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32UPDATEWINDOW);
		return BG_ERR_W32UPDATEWINDOW;
	}


	return 0;

}



//-----------------------------------------------------------------
//!    BG_WindowNullGet : return null window structure            !
//-----------------------------------------------------------------
//! return : null BG_STRUCT_WINDOW structure                      !
//-----------------------------------------------------------------
BG_STRUCT_WINDOW BG_WindowNullGet (void)
{
	BG_STRUCT_WINDOW TmpWindow;

	// no error management

	TmpWindow.sTitle = "";
	TmpWindow.hBGWnd = NULL;
	TmpWindow.pfFormProc = NULL;
	TmpWindow.iLeft = 0;
	TmpWindow.iTop = 0;
	TmpWindow.iWidth = 0;
	TmpWindow.iHeight = 0;
	TmpWindow.iXDrawPos = 0;
	TmpWindow.iYDrawPos = 0;
	TmpWindow.iLineCnt = 0;
	TmpWindow.iFirstLineCnt = 0;
	TmpWindow.iLastLineCnt = 0;
	TmpWindow.bInvisible = false;
	TmpWindow.sFontName = "";
	TmpWindow.iFontHeight = 0;
	TmpWindow.bFontBold = false;
	TmpWindow.bFontItalic = false;
	TmpWindow.bUnderline = false;
	TmpWindow.bVScrollBar = false;
	TmpWindow.iBackColor = 0;
	TmpWindow.bStatusBar = false;
	TmpWindow.bModal = false;
	TmpWindow.hStatusBar = NULL;
	TmpWindow.sStatusBar = "";
	TmpWindow.sReturn = "";

	return TmpWindow;

}



//-----------------------------------------------------------------
//!    BG_WindowGet : return information of a window              !
//-----------------------------------------------------------------
//! sBGName : name of the window                                  !
//!                                                               !
//! return : window structure or a NULL structure if fail         !
//-----------------------------------------------------------------
BG_STRUCT_WINDOW BG_WindowGet (const string sBGName)
{

	BG_STRUCT_WINDOW TmpWindow;
	BGWindowItDef BGWindowIt;


	// no error management

	// search item
	BGWindowIt = BGWindow.find (sBGName);
	if (BGWindowIt != BGWindow.end())
		TmpWindow = BGWindow [sBGName];
	else
		TmpWindow = BG_WindowNullGet ();

	return TmpWindow;

}



//-----------------------------------------------------------------
//!    BG_FormReturnValueGet : get the return value of a form     !
//-----------------------------------------------------------------
//! sBGFormName : name of the form (or window)                    !
//!																  !
//! return : return value (string)                                !
//-----------------------------------------------------------------
string BG_FormReturnValueGet (const string sBGFormName)
{

	BG_STRUCT_WINDOW TmpWindow;


	// for error management
	BG_ErrorSet (sBGFormName, "BG_FormReturnValueGet");

	// look for the window number
	TmpWindow = BG_WindowGet (sBGFormName);
	if (TmpWindow.sTitle == "")
	{
		BG_ErrorDisplay (BG_ERR_NOTOPEN);
		return "";
	}

	return TmpWindow.sReturn;

}



//-----------------------------------------------------------------
//!    BG_ResetBipGDIIntl : Reset Bip variables                   !
//-----------------------------------------------------------------
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
//! INTERNAL FUNCTION : use by BG_CreateMainWindow                !
//-----------------------------------------------------------------
int BG_ResetBipGDIIntl (void)
{
	int iError;
	INITCOMMONCONTROLSEX CommCtrl;


	// for error management
	BG_ErrorSave ();
	BG_ErrorSet ("", "BG_ResetBipGDIIntl");

	// reset window structure
	BGWindow.clear ();

	// reset current window
	BGCurWindow = "";

	// reset main window name
	BGMainWindow = "";

	// reset current form
	BGCurForm = "";

	// reset handle for the menu
	hBGMenu = NULL;

	// reset menu structure
	BGMenu.clear ();

	// reset form item structure
	BGItem.clear ();

	// get hInstance
	hBGInstance = GetModuleHandle (NULL);
	if (hBGInstance == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32UPDATEWINDOW);
		return BG_ERR_W32UPDATEWINDOW;
	}

	// store the scroll bar's size in EG variables
	BGScrollBar.iHsize = GetSystemMetrics(SM_CYHSCROLL);
	if (BGScrollBar.iHsize == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32GETSYSMETRIC);
		return BG_ERR_W32GETSYSMETRIC;
	}

	BGScrollBar.iVsize = GetSystemMetrics(SM_CYVSCROLL);
	if (BGScrollBar.iVsize == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32GETSYSMETRIC);
		return BG_ERR_W32GETSYSMETRIC;
	}

	// initialize common control (statusbar)
	CommCtrl.dwSize = sizeof (CommCtrl);
	CommCtrl.dwICC = ICC_WIN95_CLASSES;

	iError = InitCommonControlsEx (&CommCtrl);
	if (iError == FALSE)
	{
		BG_ErrorDisplay (BG_ERR_W32INITCOMMCTRL);
		return BG_ERR_W32INITCOMMCTRL;
	}


	// ResetBipGDI is done
	BG_RESETBIPGDI_DONE = true;

	// restore error message
	BG_ErrorRestore ();

	return 0;

}



//-----------------------------------------------------------------
//!    BG_MessageLoop : Standard message loop for the application !
//-----------------------------------------------------------------
UINT BG_MessageLoop (void)
{
	MSG msg;
	HWND hBGwnd;
	BG_STRUCT_WINDOW TmpWindow;


	// main message loop
	while (GetMessage (&msg, NULL, 0, 0))
	{
		// detect if a form window is active
		if (BGCurForm != "")
		{
			// get window structure
			TmpWindow = BG_WindowGet (BGCurWindow);

			// retrieve form handle
			if (TmpWindow.hBGWnd != NULL)
				hBGwnd = TmpWindow.hBGWnd;
			else
				hBGwnd = NULL;
		}
		else
			// no form is active
			hBGwnd = NULL;


		// dispatch messages
		if (!IsWindow (hBGwnd) || !IsDialogMessage (hBGwnd, &msg))
	    { 
		    TranslateMessage (&msg);
			DispatchMessage (&msg);
	    }
	}


	return msg.wParam;

}




//-----------------------------------------------------------------
//!    BG_IntToString : Convert an integer into a string          !
//-----------------------------------------------------------------
//! iInt : int number to convert                                  !
//!                                                               !
//! return : number into a string                                 !
//-----------------------------------------------------------------
string BG_IntToString (const int iInt)
{

	ostringstream sStream;

	sStream << iInt;

	return sStream.str();

}



//-----------------------------------------------------------------
//!    BG_StringToInt : Convert a string to an integer            !
//-----------------------------------------------------------------
//! sString : string contaning integer value                      !
//! iBase : base of the number into the string (default = 10)     !
//!                                                               !
//! return : integer value                                        !
//-----------------------------------------------------------------
int BG_StringToInt (const string sString, const int iBase)
				// iBase = 10
{
	int iBGint;
	int iTemp;

	// return zero if string is empty
	if (sString == "")
		return 0;

	// use decimal if base is ko
	if (iBase != 10 && iBase != 16 && iBase != 8 && iBase != 2)
		iTemp = 10;
	else
		iTemp = iBase;

	istringstream sStream (sString);
	sStream >> setbase (iTemp) >> iBGint;

	return iBGint;

}



//-----------------------------------------------------------------
//!    BG_MenuAdd : define one menu line into menu structure      !
//-----------------------------------------------------------------
//! sBGIntName : internal menu name (not null) ("-" for separator)!
//! iBGColumn : menu column number (min=1)                        !
//! iBGLine : menu line number (min=1) (1=column name)            !
//! sBGWinName : menu name (null for separator)                   !
//! sBGParam : parameters (default = "")                          !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
//! if menu line is already define, it is overwritten             !
//-----------------------------------------------------------------
int BG_MenuAdd (const string sBGIntName, const int iBGColumn, const int iBGLine,
				const string sBGWinName, const string sBGParam)
				// sBGParam = ""
{
	string sString;
	BG_STRUCT_WINDOW MainWindow;
	BG_STRUCT_MENU TmpMenu;


	// for error management
	BG_ErrorSet (sBGIntName, "BG_MenuAdd");

	// internal name is empty
	if (sBGIntName == "")
	{
		BG_ErrorDisplay (BG_ERR_MENUINTEMPTY);
		return BG_ERR_MENUINTEMPTY;
	}

	// window name is empty
	if (sBGWinName == "" && sBGIntName != "-")
	{
		BG_ErrorDisplay (BG_ERR_MENUNWINEMPTY);
		return BG_ERR_MENUNWINEMPTY;
	}

	// bad column number
	if (iBGColumn < 1 || iBGColumn > BG_MAX_MENU - 1)
	{
		BG_ErrorDisplay (BG_ERR_MENUBADCOLNUM);
		return BG_ERR_MENUBADCOLNUM;
	}

	// bad line number
	if (iBGLine < 1 || iBGLine > BG_MAX_MENU - 1)
	{
		BG_ErrorDisplay (BG_ERR_MENUBADLINENUM);
		return BG_ERR_MENUBADLINENUM;
	}

	// main window not open
	MainWindow = BG_WindowGet (BGMainWindow);
	if (MainWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_MAINNOTOPEN);
		return BG_ERR_MAINNOTOPEN;
	}


	// fill menu structure
	TmpMenu.sIntName = sBGIntName;
	TmpMenu.sWinName = sBGWinName;


	// search enable parameter
	sString = BG_StringParamGetInt (BG_PRM_ENABLE, sBGParam);
	if (BG_StringIsFalse (sString))
		TmpMenu.bEnable = false;
	else
		TmpMenu.bEnable = true;

	// search check parameter
	sString = BG_StringParamGetInt (BG_PRM_CHECK, sBGParam);
	if (BG_StringIsTrue (sString))
		TmpMenu.bCheck = true;
	else
		TmpMenu.bCheck = false;

	// save menu
	BGMenu [(iBGColumn * BG_MAX_MENU) + iBGLine] = TmpMenu;

	return 0;

}



//-----------------------------------------------------------------
//!    BG_MenuGet : return item menu structure                    !
//-----------------------------------------------------------------
//! iBGColumn : number of the column                              !
//! iBGLine : number of the line                                  !
//!                                                               !
//! return : BG_STRUCT_MENU structure (null if not found)         !
//-----------------------------------------------------------------
BG_STRUCT_MENU BG_MenuGet (const int iBGColumn, const int iBGLine)
{
	BG_STRUCT_MENU TmpMenu;
	BGMenuItDef BGMenuIt;

	// no error management

	// search menu by number
	BGMenuIt = BGMenu.find ((iBGColumn * BG_MAX_MENU) + iBGLine);
	if (BGMenuIt != BGMenu.end())
		TmpMenu = BGMenu [(iBGColumn * BG_MAX_MENU) + iBGLine];
	else
	{
		TmpMenu.sIntName = "";
		TmpMenu.sWinName = "";
		TmpMenu.bEnable = NULL;
		TmpMenu.bCheck = NULL;
	}

	return TmpMenu;

}



//-----------------------------------------------------------------
//!    BG_MenuGet : return item menu structure                    !
//-----------------------------------------------------------------
//! sBGIntName : internal name of the menu                        !
//!                                                               !
//! return : BG_STRUCT_MENU structure (null if not found)         !
//-----------------------------------------------------------------
BG_STRUCT_MENU BG_MenuGet (const string sBGIntName)
{
	BG_STRUCT_MENU TmpMenu;
	BGMenuItDef BGMenuIt;

	// no error management

	// search menu by internal name
	for (BGMenuIt = BGMenu.begin(); BGMenuIt != BGMenu.end(); BGMenuIt++)
	{
		TmpMenu = BGMenuIt->second;
		if (TmpMenu.sIntName == sBGIntName)
			break;
	}

	// return null structure
	if (BGMenuIt == BGMenu.end())
	{
		TmpMenu.sIntName = "";
		TmpMenu.sWinName = "";
		TmpMenu.bEnable = NULL;
		TmpMenu.bCheck = NULL;
	}

	return TmpMenu;

}



//-----------------------------------------------------------------
//!    BG_MenuMessage : identify the menu that has been clicked   !
//-----------------------------------------------------------------
//! message : message from messageloop                            !
//! wParam : wParam from messageloop                              !
//!																  !
//! return : internal name of menu which has been clicked         !
//!          or "" if no menu clic                                !
//-----------------------------------------------------------------
string BG_MenuMessage (const UINT message, const WPARAM wParam)
{

	int iIndex;

	// no error management

	// initialise to no clic on menu
	iIndex = -1;

	// select line number of menu
	if (message == WM_COMMAND)
		iIndex = BG_MenuNumGet (LOWORD (wParam));

	if (iIndex == -1)
		return "";
	else
		return BGMenu [iIndex].sIntName;

}



//-----------------------------------------------------------------
//!    BG_MenuDisplay : attach menu to main window and display it !
//-----------------------------------------------------------------
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_MenuDisplay (void)
{
	int iColumn;
	int iLine;
	int iWinNum;
	HMENU hSubMenu;
	UINT uDisplay;
	int iError;
	BG_STRUCT_MENU stSubMenu;
	BG_STRUCT_MENU stMenu;
	BG_STRUCT_WINDOW MainWindow;


	// for error management
	BG_ErrorSet ("", "BG_MenuDisplay");

	// main window not open
	MainWindow = BG_WindowGet (BGMainWindow);
	if (MainWindow.hBGWnd == NULL)
	{
		BG_ErrorDisplay (BG_ERR_MAINNOTOPEN);
		return BG_ERR_MAINNOTOPEN;
	}

	// delete old menu
	if (hBGMenu != NULL)
	{
		iError = DestroyMenu (hBGMenu);
		if (iError == 0)
		{
			BG_ErrorDisplay (BG_ERR_W32DESTROYMENU);
			return BG_ERR_W32DESTROYMENU;
		}

		// reset menu handle
		hBGMenu = NULL;
	}
	
	// create windows menu
	hBGMenu = CreateMenu ();
	if (hBGMenu == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32CREATEMENU);
		return BG_ERR_W32CREATEMENU;
	}

	// initialise windows menu number
	iWinNum = BG_MENU_NUM;

	for (iColumn = 1; iColumn < BG_MAX_MENU; iColumn++)
	{
		// look for submenu (column = n , line = 1)
		stSubMenu = BG_MenuGet (iColumn, 1);
		if (stSubMenu.sIntName == "")
			break;

		// create submenu (column menu)
		hSubMenu = CreateMenu ();
		if (hSubMenu == NULL)
		{
			BG_ErrorDisplay (BG_ERR_W32CREATEMENU);
			return BG_ERR_W32CREATEMENU;
		}

		// look for submenu item
		for (iLine = 2; iLine < BG_MAX_MENU; iLine++)
		{
			// look for item
			stMenu = BG_MenuGet (iColumn, iLine);
			if (stMenu.sIntName == "")
				break;

			if (stMenu.sIntName == "-")
			{
				// this is a menu separator
				iError = AppendMenu (hSubMenu, MF_SEPARATOR, 0, NULL);
				if (iError == 0)
				{
					BG_ErrorDisplay (BG_ERR_W32APPENDMENU);
					return BG_ERR_W32APPENDMENU;
				}
			}
			else
			{
				// define attribute of the menu item
				uDisplay = MF_STRING;

				// gray item
				if (!stMenu.bEnable)
					uDisplay = uDisplay | MF_GRAYED;

				// check item
				if (stMenu.bCheck)
					uDisplay = uDisplay | MF_CHECKED;

				// create submenu item
				iError = AppendMenu (hSubMenu, uDisplay, iWinNum,
									stMenu.sWinName.c_str());
				if (iError == 0)
				{
					BG_ErrorDisplay (BG_ERR_W32APPENDMENU);
					return BG_ERR_W32APPENDMENU;
				}

				// save number into menu item
				stMenu.iId = iWinNum;
				BGMenu [(iColumn * BG_MAX_MENU) + iLine] = stMenu;

				iWinNum++;
			}
		}


		// create submenu title
		iError = AppendMenu (hBGMenu, MF_POPUP, (UINT) hSubMenu,
							stSubMenu.sWinName.c_str());
		if (iError == 0)
		{
			BG_ErrorDisplay (BG_ERR_W32APPENDMENU);
			return BG_ERR_W32APPENDMENU;
		}
	}


	// attach menu to main window and display it
	iError = SetMenu (MainWindow.hBGWnd, hBGMenu);
	if (iError == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32SETMENU);
		return BG_ERR_W32SETMENU;
	}

	return 0;

}



//-----------------------------------------------------------------
//!    BG_MenuNumGet : return menu number                         !
//-----------------------------------------------------------------
//! iBGId : Id of the menu (return by message loop)               !
//!                                                               !
//! return : line number or -1 if not found                       !
//-----------------------------------------------------------------
int BG_MenuNumGet (const int iBGId)
{
	BG_STRUCT_MENU TmpMenu;
	BGMenuItDef BGMenuIt;

	// no error management

	// search menu by Id
	for (BGMenuIt = BGMenu.begin(); BGMenuIt != BGMenu.end(); BGMenuIt++)
	{
		TmpMenu = BGMenuIt->second;
		if (TmpMenu.iId == iBGId)
			break;
	}

	// return menu number or -1
	if (BGMenuIt == BGMenu.end())
		return -1;
	else
		return BGMenuIt->first;

}



//-----------------------------------------------------------------
//!    BG_MenuNumGet : return menu number                         !
//-----------------------------------------------------------------
//! sBGIntName : internal name of the menu                        !
//!                                                               !
//! return : line number or -1 if not found                       !
//-----------------------------------------------------------------
int BG_MenuNumGet (const string sBGIntName)
{
	BG_STRUCT_MENU TmpMenu;
	BGMenuItDef BGMenuIt;

	// no error management

	// search menu by internal name
	for (BGMenuIt = BGMenu.begin(); BGMenuIt != BGMenu.end(); BGMenuIt++)
	{
		TmpMenu = BGMenuIt->second;
		if (TmpMenu.sIntName == sBGIntName)
			break;
	}

	// return menu number or -1
	if (BGMenuIt == BGMenu.end())
		return -1;
	else
		return BGMenuIt->first;

}



//-----------------------------------------------------------------
//!    BG_MenuItemEnable : enable or disable menu item            !
//-----------------------------------------------------------------
//! sBGIntName : internal menu name (not null)                    !
//! bBGEnable : true if menu must be enable                       !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_MenuItemEnable (const string sBGIntName, const bool bBGEnable)
{
	UINT uDisplay;
	int iError;
	BG_STRUCT_MENU TmpMenu;


	// for error management
	BG_ErrorSet (sBGIntName, "BG_MenuItemEnable");

	// item name is empty
	if (sBGIntName == "")
	{
		BG_ErrorDisplay (BG_ERR_MENUITEMEMPTY);
		return BG_ERR_MENUITEMEMPTY;
	}

	// search for item
	TmpMenu = BG_MenuGet (sBGIntName);
	if (TmpMenu.sIntName == "")
	{
		BG_ErrorDisplay (BG_ERR_MENUITEMNOTFOUND);
		return BG_ERR_MENUITEMNOTFOUND;
	}


	// modify item
	if (TmpMenu.bEnable != bBGEnable)
	{
		TmpMenu.bEnable = bBGEnable;

		uDisplay = MF_BYCOMMAND | MF_ENABLED;
		if (!bBGEnable)
			uDisplay = MF_BYCOMMAND | MF_GRAYED;

		iError = EnableMenuItem (hBGMenu, TmpMenu.iId, uDisplay);
		if (iError == -1)
		{
			BG_ErrorDisplay (BG_ERR_W32ENABLEMENUIT);
			return BG_ERR_W32ENABLEMENUIT;
		}

		// save menu
		BGMenu [BG_MenuNumGet (sBGIntName)] = TmpMenu;
	}

	return 0;

}



//-----------------------------------------------------------------
//!    BG_MenuItemCheck : check or uncheck menu item              !
//-----------------------------------------------------------------
//! sBGIntName : internal menu name (not null)                    !
//! bBGCheck : true if menu must be check                         !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_MenuItemCheck (const string sBGIntName, const bool bBGCheck)
{
	UINT uDisplay;
	int iError;
	BG_STRUCT_MENU TmpMenu;


	// for error management
	BG_ErrorSet (sBGIntName, "BG_MenuItemCheck");

	// item name is empty
	if (sBGIntName == "")
	{
		BG_ErrorDisplay (BG_ERR_MENUITEMEMPTY);
		return BG_ERR_MENUITEMEMPTY;
	}

	// search for item
	TmpMenu = BG_MenuGet (sBGIntName);
	if (TmpMenu.sIntName == "")
	{
		BG_ErrorDisplay (BG_ERR_MENUITEMNOTFOUND);
		return BG_ERR_MENUITEMNOTFOUND;
	}

	// modify item
	if (TmpMenu.bCheck != bBGCheck)
	{
		TmpMenu.bCheck = bBGCheck;

		uDisplay = MF_BYCOMMAND | MF_CHECKED;
		if (!bBGCheck)
			uDisplay = MF_BYCOMMAND | MF_UNCHECKED;

		iError = CheckMenuItem (hBGMenu, TmpMenu.iId, uDisplay);
		if (iError == -1)
		{
			BG_ErrorDisplay (BG_ERR_W32CHECKMENUIT);
			return BG_ERR_W32CHECKMENUIT;
		}

		// save menu
		BGMenu [BG_MenuNumGet (sBGIntName)] = TmpMenu;
	}


	return 0;

}



//-----------------------------------------------------------------
//!    BG_StringParamGetInt : return value of a parameter         !
//-----------------------------------------------------------------
//! sBGParam : name of the parameter                              !
//! sBGString : string containing all parameters                  !
//!                                                               !
//! return : value of the param or "" if error or not found       !
//-----------------------------------------------------------------
//! - search is not case sensitive                                !
//! - only the first occurence of the parameter is return         !
//! - format of sBG_String : "param1=value1;param2=value2; ..."   !
//!     - character between param and value -> "="                !
//!     - character between parameters -> ";"                     !
//-----------------------------------------------------------------
//!  INTERNAL FUNCTION                                            !
//-----------------------------------------------------------------
string BG_StringParamGetInt (const string sBGParam, const string sBGString)
{
	int iBegin;
	int iEnd;
	string sParam;
	string sString;
	string sTemp;

	// no error management

	// end if one input string is empty
	if (sBGParam == "" || sBGString == "")
		return "";

	// tranfert into local and upper case string 
	sParam = BG_StringToUpper (sBGParam);
	sString = BG_StringToUpper (sBGString);

	// search for parameter
	iBegin = -1;
	do
		iBegin = sString.find (sParam + "=", iBegin + 1);
	while (iBegin > 0 && sString [iBegin - 1] != ';');

	if (iBegin < 0)
	{
		// param not found
		return "";
	}
	else
	{
		//select and return value
		iBegin += sParam.length ();
		iBegin += 1;

		iEnd = sString.find (";", iBegin);
		if (iEnd < 0)
			iEnd = sString.length () + iBegin;

		sTemp = sBGString.substr (iBegin, iEnd - iBegin);

		// delete leading and ending spaces
		iBegin = sTemp.find_first_not_of (" ");
		iEnd = sTemp.find_last_not_of (" ");

		return sTemp.substr (iBegin, iEnd - iBegin + 1);
	}
}



//-----------------------------------------------------------------
//!    BG_StringIsTrue : tell if a string is consider as true     !
//-----------------------------------------------------------------
//! sBGInput : input string                                       !
//!                                                               !
//! return : true or false                                        !
//-----------------------------------------------------------------
//! string content is consider true, if it is equal to :          !
//!  (not case sensitive) true,t,yes,y                            !
//! in all other case (ie string is empty) content is consider    !
//!     false                                                     !
//-----------------------------------------------------------------
bool BG_StringIsTrue (const string sBGInput)
{
	string sString;

	sString = BG_StringToUpper (sBGInput);

	if (sString == "TRUE" || sString == "T" || sString == "YES" || sString == "Y")
		return true;
	else
		return false;

}



//-----------------------------------------------------------------
//!    BG_StringIsFalse : tell if a string is consider as false   !
//-----------------------------------------------------------------
//! sBGInput : input string                                       !
//!                                                               !
//! return : true or false                                        !
//-----------------------------------------------------------------
//! string content is consider false, if it is equal to :         !
//!  (not case sensitive) false,f,no,n                            !
//! in all other case (ie string is empty) content is consider    !
//!     false                                                     !
//-----------------------------------------------------------------
bool BG_StringIsFalse (const string sBGInput)
{
	string sString;

	sString = BG_StringToUpper (sBGInput);

	if (sString == "FALSE" || sString == "F" || sString == "NO" || sString == "N")
		return true;
	else
		return false;

}



//-----------------------------------------------------------------
//!    BG_ColorNumGet : return int value of a color               !
//-----------------------------------------------------------------
//! sBGColor : define color or hexa value                         !
//!                                                               !
//! return : int color                                            !
//-----------------------------------------------------------------
int BG_ColorNumGet (const string sBGColor)
{

	string sColor;
	int iTemp;

	// convert input string to upper case
	sColor = BG_StringToUpper (sBGColor);

	if (sColor == "BLACK")
		return 0x00000000;
	else if (sColor == "WHITE")
		return 0x00FFFFFF;
	else if (sColor == "RED")
		return 0x000000FF;
	else if (sColor == "BLUE")
		return 0x00FF0000;
	else if (sColor == "GREEN")
		return 0x0000FF00;
	else if (sColor == "YELLOW")
		return 0x0000FFFF;
	else if (sColor == "MAGENTA")
		return 0x00FF00FF;
	else if (sColor == "CYAN")
		return 0x00FFFF00;
	else if (sColor == "GREY")
		return 0x00BFBFBF;
	else if (sColor == "LIGHTGREY")
		return 0x00DFDFDF;
	else if (sColor == "DARKGREY")
		return 0x007F7F7F;
	else
	{
		// convert hexa color
		iTemp = BG_StringToInt (sBGColor, 16);
		return iTemp;
	}

}



//-----------------------------------------------------------------
//!    BG_FormItemAdd : declare one item for a form window        !
//-----------------------------------------------------------------
//! sBGFormName : internal name of the form window                !
//! sBGItemName : internal name of the item (widget)              !
//! sBGItemType : type of item (STATIC,EDIT,BUTTON, ...)          !
//! iBGOrder : order number of item (must be unique)              !
//! iBGLeft : left position of the item in pixel                  !
//! iBGTop : top position of the item in pixel                    !
//! iBGWidth : width of the item in pixel                         !
//! iBGHeight : height of the item in pixel                       !
//! sBGParam : parameters (default = "")                          !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
//! if item is already define, it is overwritten                  !
//-----------------------------------------------------------------
int BG_FormItemAdd (const string sBGFormName, const string sBGItemName,
					const string sBGItemType, const int iBGOrder,
					const int iBGLeft, const int iBGTop, const int iBGWidth,
					const int iBGHeight, const string sBGParam)
					// sBGParam = ""
{
	string sValue;
	string sButton;
	string sType;
	string sRemark;
	string sFormat;
	int iHpos;
	int iVpos;
	string sFtName;
	int iFtSize;
	bool bFtBold;
	bool bFtItalic;
	bool bFtUline;
	string sString;
	bool bNotNull;
	bool bEdit;
	bool bCheck;
	bool bEnable;
	bool bVisible;
	int iLength;
	int iBackColor;
	int iTextColor;
	int iBrdSize;
	int iBrdColor;
	string sBrdStyle;
	BG_STRUCT_FORMITEM TmpItem;


	// for error management
	BG_ErrorSet (sBGFormName + "." + sBGItemName, "BG_FormItemAdd");

	// internal child window name is empty
	if (sBGFormName == "")
	{
		BG_ErrorDisplay (BG_ERR_BADWINDNAME);
		return BG_ERR_BADWINDNAME;
	}

	// internal item name is empty
	if (sBGItemName == "")
	{
		BG_ErrorDisplay (BG_ERR_FORMITEMNAMEBAD);
		return BG_ERR_FORMITEMNAMEBAD;
	}


	// verify if item already exists
	TmpItem = BG_FormItemGet (sBGFormName, sBGItemName);
	if (TmpItem.sType != "")
	{
		BG_ErrorDisplay (BG_ERR_FORMITEMREDEDINE);
		return BG_ERR_FORMITEMREDEDINE;
	}

	// create item
	if (sBGItemType == BG_ITEM_STATIC || sBGItemType == BG_ITEM_RECT ||
		sBGItemType == BG_ITEM_EDIT || sBGItemType == BG_ITEM_BUTTONSYS ||
		sBGItemType == BG_ITEM_BUTTON || sBGItemType == BG_ITEM_CHECKSYS ||
		sBGItemType == BG_ITEM_COMBOSYS || sBGItemType == BG_ITEM_DATAGRID)
	{
		// search default value parameter
		sValue = BG_StringParamGetInt (BG_PRM_VALUE, sBGParam);

		// search button parameter
		sButton = BG_StringParamGetInt (BG_PRM_BUTTON, sBGParam);
		sButton = BG_StringToUpper (sButton);

		// search remark parameter
		sRemark = BG_StringParamGetInt (BG_PRM_REMARK, sBGParam);

		// search format parameter
		sFormat = BG_StringParamGetInt (BG_PRM_FORMAT, sBGParam);

		// search hpos parameter (default = left)
		sString = BG_StringParamGetInt (BG_PRM_TXHPOS, sBGParam);
		sString = BG_StringToUpper (sString);
		if (sString == "CENTER")
			iHpos = 0;
		else if (sString == "RIGHT")
			iHpos = 1;
		else
			iHpos = -1;

		// search vpos parameter (default = top)
		sString = BG_StringParamGetInt (BG_PRM_TXVPOS, sBGParam);
		sString = BG_StringToUpper (sString);
		if (sString == "CENTER")
			iVpos = 0;
		else if (sString == "BOTTOM")
			iVpos = 1;
		else
			iVpos = -1;
		
		// search font name parameter
		sFtName = BG_StringParamGetInt (BG_PRM_FTNAME, sBGParam);

		// search font size parameter
		sString = BG_StringParamGetInt (BG_PRM_FTSIZE, sBGParam);
		iFtSize = BG_StringToInt (sString);
		if (iFtSize < 1 || iFtSize > 256)
			iFtSize = 10;

		// search bold font parameter
		sString = BG_StringParamGetInt (BG_PRM_FTBOLD, sBGParam);
		if (BG_StringIsTrue (sString))
			bFtBold = true;
		else
			bFtBold = false;

		// search italic font parameter
		sString = BG_StringParamGetInt (BG_PRM_FTITALIC, sBGParam);
		if (BG_StringIsTrue (sString))
			bFtItalic = true;
		else
			bFtItalic = false;

		// search underline font parameter
		sString = BG_StringParamGetInt (BG_PRM_FTULINE, sBGParam);
		if (BG_StringIsTrue (sString))
			bFtUline = true;
		else
			bFtUline = false;

		// search length parameter
		sString = BG_StringParamGetInt (BG_PRM_LENGTH, sBGParam);
		if (sString == "")
			iLength = 0;
		else
			iLength = BG_StringToInt (sString);

		// search notnull parameter
		sString = BG_StringParamGetInt (BG_PRM_NOTNULL, sBGParam);
		if (BG_StringIsTrue (sString))
			bNotNull = true;
		else
			bNotNull = false;

		// search background color parameter
		sString = BG_StringParamGetInt (BG_PRM_BACKCOLOR, sBGParam);
		if (sString == "")
			iBackColor = -1;
		else
			iBackColor = BG_ColorNumGet (sString);

		// search text color parameter
		sString = BG_StringParamGetInt (BG_PRM_TEXTCOLOR, sBGParam);
		if (sString == "")
			iTextColor = -1;
		else
			iTextColor = BG_ColorNumGet (sString);

		// search border type parameter
		sBrdStyle = BG_StringParamGetInt (BG_PRM_BRDSTYLE, sBGParam);
		sBrdStyle = BG_StringToUpper (sBrdStyle);
		if (sBrdStyle == "")
			sBrdStyle = "SOLID";
		if (sBrdStyle != "SOLID" && sBrdStyle != "3DOUT" && sBrdStyle != "3DIN")
		{
			BG_ErrorDisplay (BG_ERR_RECTBADSTYLE);
			return BG_ERR_RECTBADSTYLE;
		}

		// search border size parameter
		sString = BG_StringParamGetInt (BG_PRM_BRDSIZE, sBGParam);
		iBrdSize = BG_StringToInt (sString);
		if (iBrdSize < 0 || iBrdSize > 16)
			iBrdSize = 0;

		// search border color parameter
		sString = BG_StringParamGetInt (BG_PRM_BRDCOL, sBGParam);
		if (sString == "")
			iBrdColor = 0;
		else
			iBrdColor = BG_ColorNumGet (sString);

		// search type parameter
		sType = BG_StringParamGetInt (BG_PRM_TYPE, sBGParam);
		sType = BG_StringToUpper (sType);
		if (sType != "" && sType != BG_RET_STRING && sType != BG_RET_DATE &&
			sType != BG_RET_CURRENCY)
		{
			BG_ErrorDisplay (BG_ERR_FORMITEMRETBAD);
			return BG_ERR_FORMITEMRETBAD;
		}

		// default type is STRING for EDIT and COMBOSYS item
		if ((sBGItemType == BG_ITEM_EDIT || sBGItemType == BG_ITEM_COMBOSYS) && sType == "")
			sType = BG_RET_STRING;

		
		// type STRING,DATE,CURRENCY are for EDIT and COMBOSYS item
		if (sBGItemType != BG_ITEM_EDIT && sBGItemType != BG_ITEM_COMBOSYS && 
			(sType == BG_RET_STRING || sType == BG_RET_DATE || sType == BG_RET_CURRENCY))
		{
			BG_ErrorDisplay (BG_ERR_FORMBADSTRING);
			return BG_ERR_FORMBADSTRING;
		}

		// date need a format
		if (sBGItemType == BG_ITEM_EDIT && sType == BG_RET_DATE && sFormat == "")
		{
			BG_ErrorDisplay (BG_ERR_FORMDTFMTBAD);
			return BG_ERR_FORMDTFMTBAD;
		}
		
		// currency need a format
		if (sBGItemType == BG_ITEM_EDIT && sType == BG_RET_CURRENCY && sFormat == "")
		{
			BG_ErrorDisplay (BG_ERR_FORMDTFMTBAD);
			return BG_ERR_FORMDTFMTBAD;
		}

		// search edit parameter
		sString = BG_StringParamGetInt (BG_PRM_EDIT, sBGParam);
		if (BG_StringIsTrue (sString))
			bEdit = true;
		else
			bEdit = false;

		// search check parameter
		sString = BG_StringParamGetInt (BG_PRM_CHECK, sBGParam);
		if (BG_StringIsTrue (sString))
			bCheck = true;
		else
			bCheck = false;

		// search enable parameter
		sString = BG_StringParamGetInt (BG_PRM_ENABLE, sBGParam);
		if (BG_StringIsFalse (sString))
			bEnable = false;
		else
			bEnable = true;

		// search visible parameter
		sString = BG_StringParamGetInt (BG_PRM_VISIBLE, sBGParam);
		if (BG_StringIsFalse (sString))
			bVisible = false;
		else
			bVisible = true;


		// save item
		TmpItem.sType = sBGItemType;
		TmpItem.sFormat = sFormat;
		TmpItem.iLength = iLength;
		TmpItem.iOrder = iBGOrder;
//		TmpItem.hHandle = hHandle;
		TmpItem.sValue = sValue;
		TmpItem.bEdit = bEdit;
		TmpItem.bCheck = bCheck;
		TmpItem.bEnable = bEnable;
		TmpItem.bVisible = bVisible;
		TmpItem.bNotNull = bNotNull;
		TmpItem.iBackColor = iBackColor;
		TmpItem.iTextColor = iTextColor;
		TmpItem.hBackBrush = NULL;
		TmpItem.hTextBrush = NULL;
		TmpItem.iId = 0;
		TmpItem.iLeft = iBGLeft;
		TmpItem.iTop = iBGTop;
		TmpItem.iWidth = iBGWidth;
		TmpItem.iHeight = iBGHeight;
		TmpItem.sButton = sButton;
		TmpItem.sRemark = sRemark;
		TmpItem.sRetType = sType;
		TmpItem.iTextHPos = iHpos;
		TmpItem.iTextVPos = iVpos;
		TmpItem.sFtName = sFtName;
		TmpItem.iFtSize = iFtSize;
		TmpItem.bFtBold = bFtBold;
		TmpItem.bFtItalic = bFtItalic;
		TmpItem.bFtUline = bFtUline;
		TmpItem.iBrdSize = iBrdSize;
		TmpItem.iBrdColor = iBrdColor;
		TmpItem.sBrdStyle = sBrdStyle;
		TmpItem.sParam = sBGParam;
		TmpItem.bReturn = false;
		TmpItem.sReturn = "";

		BGItem [sBGFormName + "." + sBGItemName] = TmpItem;
	}
	else
	{
		BG_ErrorDisplay (BG_ERR_FORMITEMTYPEBAD);
		return BG_ERR_FORMITEMTYPEBAD;
	}


	return 0;

}



//-----------------------------------------------------------------
//!    BG_FormProcIntl : Window Procedure for form window         !
//-----------------------------------------------------------------
//! hBGCurWnd : current window handle                             !
//! message : message from message loop                           !
//! wParam : wParam from message loop                             !
//! lParam : lParam from message loop                             !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
//! INTERNAL FUNCTION : use by BG_FormCreate                      !
//-----------------------------------------------------------------
BOOL CALLBACK BG_FormProcIntl (HWND hBGCurWnd, UINT message, WPARAM wParam, LPARAM lParam)
// laisser BOOL en majuscule
{
	int iItem;
	int iTemp;
	int iTmpFlag;
	int iError;
	TCHAR tChar1024 [1024];
	UINT uDrawMode;
	BP_Date dateTemp;
	BP_Currency curTemp;
	HBRUSH hWindowBgBrush;			// backgroung brush
	LPDRAWITEMSTRUCT lpItem;
	RECT rcSize;
	COLORREF clColor;
	string sErrMsg [10];
	string sString;
	BG_STRUCT_WINDOW TmpWindow;
	BG_STRUCT_FORMITEM TmpItem;
	BGItemItDef BGItemIt;
	string sFormName;
	string sItemName;

	// for error management
	BG_ErrorSet ("", "BG_FormProcIntl");


	// initialize variables
	hWindowBgBrush = NULL;

//##############################
sErrMsg [0] = "Saisie obligatoire";
sErrMsg [1] = "Saisie incorrecte";
//##############################


	// get window name
	sFormName = BGCurForm;


	// get window structure
	TmpWindow = BG_WindowGet (sFormName);
	if (TmpWindow.sTitle == "")
		return BG_ERR_NOTOPEN;

	// retrieve item name
	if (sFormName != "")
		sItemName = BG_FormItemNameGet (sFormName, (HWND) lParam);
	else
		sItemName = "";

	// call user procedure
	if (TmpWindow.pfFormProc != NULL)
	{
		iError = TmpWindow.pfFormProc (sFormName, sItemName, BG_MSG_BEFORE,
							hBGCurWnd, message, wParam, lParam);
		// stop procedure, if user has process the message
		if (iError != 0)
			return iError;
	}


	switch (message)
	{
		case WM_INITDIALOG:
		{
			// create all items into the form, order by iOrder
			for (iItem = 1; true; iItem++)
			{
				// flag : =1 if end of item
				iTmpFlag = 0;

				for (BGItemIt = BGItem.begin(); BGItemIt != BGItem.end(); BGItemIt++)
				{
					sString = BGItemIt->first;
					TmpItem = BGItemIt->second;
					iTemp = sString.find (".");

					// stop if no separator found
					if (iTemp < 1)
						break;

					// create item if found
					sItemName = sString.substr (iTemp + 1, sString.length());
					sString = sString.substr (0, iTemp);
					if (sString == sFormName && TmpItem.iOrder == iItem)
					{
						iError = BG_FormCreateItem (hBGCurWnd, sFormName, sItemName);
						iTmpFlag = 1;
						break;
					}
				}

				// end if next item doesn't exists
				if (iTmpFlag == 0)
					break;
			}

			// create status bar
			if (TmpWindow.bStatusBar)
			{
				TmpWindow.hStatusBar = CreateWindowEx (0, STATUSCLASSNAME,
					(LPCTSTR) NULL, SBARS_SIZEGRIP | WS_CHILD | WS_VISIBLE,
					0, 0, 0, 0, hBGCurWnd, (HMENU) 100, hBGInstance, NULL);

				// save window structure info
				BGWindow [sFormName] = TmpWindow;
			}

			return TRUE;
		}


		// size of window has change
		case WM_SIZE:
		{
			if (TmpWindow.sTitle != "")
			{
				// redraw status bar
				if (TmpWindow.bStatusBar)
				{
					SendMessage (TmpWindow.hStatusBar, WM_SIZE, 0, 0);
					return TRUE;
				}
			}
		}


		// dialog box background color
		case WM_CTLCOLORDLG:
		{
			if (TmpWindow.sTitle != "")
			{
				// create background brush
				hWindowBgBrush = CreateSolidBrush (TmpWindow.iBackColor);
				if (hWindowBgBrush == NULL)
				{
					BG_ErrorDisplay (BG_ERR_W32CREATESBRUSH);
//					return BG_ERR_W32CREATESBRUSH ;
				}

				return (BOOL) hWindowBgBrush;
			}
		}


		// EDIT item attributes
		case WM_CTLCOLOREDIT:
		{
			if (TmpWindow.sTitle != "")
			{
				if (sItemName != "")
				{
					// get item structure
					TmpItem = BG_FormItemGet (sFormName, sItemName);

					// use black color if item text color not define
					if (TmpItem.iTextColor == -1)
						iTemp = BG_ColorNumGet ("black");
					else
						iTemp = TmpItem.iTextColor;
				
					// create text brush
					TmpItem.hTextBrush = CreateSolidBrush (iTemp);
					if (TmpItem.hTextBrush == NULL)
						BG_ErrorDisplay (BG_ERR_W32CREATESBRUSH);

					// set text color
					clColor = SetTextColor ((HDC) wParam, iTemp);
					if (clColor == CLR_INVALID)
						BG_ErrorDisplay (BG_ERR_W32SETBKCOLOR);

					// use white color if item back color not define
					if (TmpItem.iBackColor == -1)
						iTemp = BG_ColorNumGet ("white");
					else
						iTemp = TmpItem.iBackColor;

					// create background brush
					TmpItem.hBackBrush = CreateSolidBrush (iTemp);
					if (TmpItem.hBackBrush == NULL)
						BG_ErrorDisplay (BG_ERR_W32CREATESBRUSH);

					// set background color
					clColor = SetBkColor ((HDC) wParam, iTemp);
					if (clColor == CLR_INVALID)
						BG_ErrorDisplay (BG_ERR_W32SETBKCOLOR);


					// save structure
					BGItem [sFormName + "." + sItemName] = TmpItem;

					return (BOOL) TmpItem.hBackBrush;
				}
			}
		}


		// OWNERDRAW item attributes
		case WM_DRAWITEM:
		{
			if (TmpWindow.sTitle != "")
			{
				// get item info
				lpItem = (LPDRAWITEMSTRUCT) lParam;

				// retrieve item number (STATIC item need this)
				sItemName = BG_FormItemNameGet (sFormName, lpItem->hwndItem);
				if (sItemName != "")
				{
					// get item structure
					TmpItem = BG_FormItemGet (sFormName, sItemName);

					if (TmpItem.sType == BG_ITEM_BUTTON || TmpItem.sType == BG_ITEM_STATIC ||
						TmpItem.sType == BG_ITEM_RECT)
					{
						// use black color if text color is not define
						if (TmpItem.iTextColor == -1)
							iTemp = BG_ColorNumGet ("black");
						else
							iTemp = TmpItem.iTextColor;

						// use grey color for text if button is not enable
						if (TmpItem.sType == BG_ITEM_BUTTON && !TmpItem.bEnable)
							iTemp = BG_ColorNumGet ("grey");

						// set text color
						clColor = SetTextColor (lpItem->hDC, iTemp);
						if (clColor == CLR_INVALID)
							BG_ErrorDisplay (BG_ERR_W32SETTEXTCOLOR);

						// compute background color
						if (TmpItem.sType == BG_ITEM_BUTTON)
						{
							// use standard button color if background color is not define
							if (TmpItem.iBackColor == -1)
								iTemp = GetSysColor (COLOR_BTNFACE);
							else
								iTemp = TmpItem.iBackColor;
						}
						else
						{
							// use background color of form if not define
							if (TmpItem.iBackColor == -1)
								iTemp = TmpWindow.iBackColor;
							else
								iTemp = TmpItem.iBackColor;
						}

						// set background color
						clColor = SetBkColor (lpItem->hDC, iTemp);
						if (clColor == CLR_INVALID)
							BG_ErrorDisplay (BG_ERR_W32SETBKCOLOR);


						// select font and save previous one
//static HFONT hPolice=CreateFont(40,0,0,0,400,0,0,0,0,0,0,0,0,"Arial");
//HFONT hOrigi=(HFONT)SelectObject (lpItem->hDC,hPolice);
//SelectObject (lpItem->hDC,hOrigi);
						if (TmpItem.sFtName != "")
						{
//iBGCurWindow = iWindow;
BG_FormItemFontSet (lpItem->hDC, TmpItem.sFtName, TmpItem.iFtSize, TmpItem.bFtBold,
					TmpItem.bFtItalic, TmpItem.bFtUline);
						}


						// horizontal position for text
						if (TmpItem.iTextHPos == 0 || TmpItem.sType == BG_ITEM_BUTTON)
						{
							// center align
							uDrawMode = DT_CENTER;
						}
						else if (TmpItem.iTextHPos == -1)
						{
							// left align
							uDrawMode = DT_LEFT;
						}
						else
						{
							// right align
							uDrawMode = DT_RIGHT;
						}

						// vertical position for text
						if (TmpItem.iTextVPos == 0 || TmpItem.sType == BG_ITEM_BUTTON)
						{
							// center align
							uDrawMode = uDrawMode | DT_VCENTER;
						}
						else if (TmpItem.iTextVPos == 1)
						{
							// bottom align
							uDrawMode = uDrawMode | DT_BOTTOM | DT_SINGLELINE;
						}
						else
						{
							// top align
//							uDrawMode = uDrawMode | DT_TOP;
						}

						// define position of the border
						rcSize.left = 0;
						rcSize.top = 0;
						rcSize.right = TmpItem.iWidth;
						rcSize.bottom = TmpItem.iHeight;


						// draw the border
						if (TmpItem.sType == BG_ITEM_BUTTON)
						{
							// only for button
							if (lpItem->itemState & ODS_SELECTED)
								sString = "3DIN";
							else
								sString = "3DOUT";
						}
						else
						{
							// set border size for non solid rectangle
							if (TmpItem.sBrdStyle != "SOLID")
								TmpItem.iBrdSize = 2;

							sString = TmpItem.sBrdStyle;
						}

						// draw rectangle if size is not null
						if (sString != "SOLID" || TmpItem.iBrdSize > 0)
						{
							iError = BG_RectangleDraw (lpItem->hDC, rcSize.left, rcSize.top,
									rcSize.right, rcSize.bottom, sString,
									TmpItem.iBrdSize, TmpItem.iBrdColor,
									TmpItem.iBackColor);
							if (iError != 0)
								BG_ErrorDisplay (iError);
						}


						// modify position for the text
						rcSize.left = TmpItem.iBrdSize;
						rcSize.top = TmpItem.iBrdSize;
						rcSize.right = TmpItem.iWidth - TmpItem.iBrdSize;
						rcSize.bottom = TmpItem.iHeight - TmpItem.iBrdSize;


						// if vertical center or button
						if (TmpItem.sType == BG_ITEM_BUTTON || TmpItem.iTextVPos == 0)
							uDrawMode = uDrawMode | DT_SINGLELINE;
						else
							uDrawMode = uDrawMode | DT_WORDBREAK;

						// draw text
						if (TmpItem.sType != BG_ITEM_RECT)
						{
							// use item name if value not define for button
							if (TmpItem.sType == BG_ITEM_BUTTON && TmpItem.sValue == "")
								sString = sItemName;
							else
								sString = TmpItem.sValue;

							// draw the text
							if (sString != "")
							{
								iError = DrawText (lpItem->hDC, (LPTSTR) sString.c_str(),
													sString.length(), &rcSize, uDrawMode);
								if (iError == 0)
									BG_ErrorDisplay (BG_ERR_W32DRAWTEXT);
							}
						}

						// draw button edge
//						if (TmpItem.sType == BG_ITEM_BUTTON)
//						{
//							iError = DrawEdge (lpItem->hDC, &lpItem->rcItem, EDGE_RAISED, BF_RECT);
//							if (iError == 0)
//								BG_ErrorDisplay (BG_ERR_W32DRAWEDGE);
//						}

						// draw or delete focus rect
						if (lpItem->itemAction & ODA_FOCUS)
						{
							if (TmpItem.sType == BG_ITEM_BUTTON)
//							if (lpItem->itemState & ODS_FOCUS && TmpItem.sType == BG_ITEM_BUTTON)
							{
								// compute focus size
								rcSize = lpItem->rcItem;
								rcSize.top = rcSize.top + 5;
								rcSize.left = rcSize.left + 5;
								rcSize.right = rcSize.right - 4;
								rcSize.bottom = rcSize.bottom - 4;

								// verify values
								if (rcSize.top < 1)
									rcSize.top = 1;
								if (rcSize.left < 1)
									rcSize.left = 1;
								if (rcSize.right < 1)
									rcSize.right = lpItem->rcItem.right;
								if (rcSize.bottom < 1)
									rcSize.bottom = lpItem->rcItem.bottom;

								// draw focus rect
								iError = DrawFocusRect (lpItem->hDC, &rcSize);
								if (iError == 0)
									BG_ErrorDisplay (BG_ERR_W32DRAWFOCUSRECT);


								// display remark message
								if (TmpWindow.bStatusBar)
								{
									if (TmpWindow.sStatusBar == "")
									{
										// display default remark
										SendMessage (TmpWindow.hStatusBar, WM_SETTEXT, 0,
											(LPARAM) TmpItem.sRemark.c_str());
									}
									else
									{
										// display remark message
										SendMessage (TmpWindow.hStatusBar, WM_SETTEXT, 0,
											(LPARAM) TmpWindow.sStatusBar.c_str());
									}
								}
							}
						}
						
						return TRUE;
					}
				}
			}
		}


		case WM_COMMAND:
		{
			// current item
			iItem = LOWORD (wParam);

			if (sItemName != "")
				TmpItem = BG_FormItemGet (sFormName, sItemName);
			else
				TmpItem = BG_FormItemNullGet ();

			switch (HIWORD (wParam))
			{
				case EN_KILLFOCUS:	// edit box
				case CBN_KILLFOCUS:	// combo box
				case CBN_SELCHANGE:	// combo box
				{
					// erase remark
					TmpWindow.sStatusBar = "";

					// retrieve item value
					if (TmpItem.sRetType == BG_RET_STRING ||
						TmpItem.sRetType == BG_RET_DATE ||
						TmpItem.sRetType == BG_RET_CURRENCY)
					{
						GetDlgItemText (hBGCurWnd, iItem, tChar1024, sizeof (tChar1024));
						TmpItem.sReturn = tChar1024;
					}

					// verify STRING item
					if (TmpItem.sRetType == BG_RET_STRING)
					{
						// truncate string to length param
						if (TmpItem.iLength > 0)
							TmpItem.sReturn =
								TmpItem.sReturn.substr (0, TmpItem.iLength);

						// re-display string	
						SetDlgItemText (hBGCurWnd, iItem, TmpItem.sReturn.c_str());

						// verify if value is required
						if (TmpItem.bNotNull && TmpItem.sReturn == "")
						{
							TmpWindow.sStatusBar = sErrMsg [0];

							// notnull set : stay in the item
							SetFocus ((HWND) lParam);
						}

					}

					// verify DATE item
					if (TmpItem.sRetType == BG_RET_DATE)
					{
						dateTemp.setformat (TmpItem.sFormat);
						dateTemp.set (TmpItem.sReturn);
						iError = dateTemp.geterror();

						// display date with format
						if (iError == 0)
							SetDlgItemText (hBGCurWnd, iItem, dateTemp.str().c_str());
						else
						{
							// date is not empty
							if (iError != 8)
							{
								// stay in the item
								SetFocus ((HWND) lParam);
							}
							else
							{
								// date is empty but must not be
								if (TmpItem.bNotNull)
								{
									TmpWindow.sStatusBar = sErrMsg [0];

									// notnull set : stay in the item
									SetFocus ((HWND) lParam);
								}
							}
						}

						// save date
						TmpItem.dtReturn = dateTemp;
					}

					// verify CURRENCY item
					if (TmpItem.sRetType == BG_RET_CURRENCY)
					{
						curTemp.setformat (TmpItem.sFormat);
						curTemp.set (TmpItem.sReturn);
						iError = curTemp.geterror();

						// display currency with format
						if (iError == 0)
							SetDlgItemText (hBGCurWnd, iItem, curTemp.str().c_str());
						else
						{
							TmpWindow.sStatusBar = sErrMsg [1];

							// currency error : stay in the item
							SetFocus ((HWND) lParam);
						}

						// verify if value is required
						if (TmpItem.bNotNull && curTemp.isnull ())
						{
							TmpWindow.sStatusBar = sErrMsg [0];

							// notnull set : stay in the item
							SetFocus ((HWND) lParam);
						}

						// save currency
						TmpItem.curReturn = curTemp;

					}

					// save structure
					BGWindow [sFormName] = TmpWindow;
					BGItem [sFormName + "." + sItemName] = TmpItem;


					// call user procedure
					if (TmpWindow.pfFormProc != NULL)
					{
						// if item has has been modified
						if (sBGItemOldValue != TmpItem.sReturn ||
							bBGItemOldValue != TmpItem.bReturn)
						{
							iError = TmpWindow.pfFormProc (sFormName, sItemName, BG_MSG_ITEMCHANGE,
											hBGCurWnd, message, wParam, lParam);
							// stop procedure, if user has process the message
							if (iError != 0)
								return iError;
						}

						iError = TmpWindow.pfFormProc (sFormName, sItemName, BG_MSG_ITEMEXIT,
										hBGCurWnd, message, wParam, lParam);
						// stop procedure, if user has process the message
						if (iError != 0)
							return iError;
					}

					return TRUE;
				}

				// item receive focus
				case EN_SETFOCUS:
				case BN_SETFOCUS:
				case WM_SETFOCUS:
				case CBN_SETFOCUS:
				{
					// display remark message
					if (TmpWindow.bStatusBar)
					{
						if (TmpWindow.sStatusBar == "")
						{
							// display default remark
							SendMessage (TmpWindow.hStatusBar, WM_SETTEXT, 0,
								(LPARAM) TmpItem.sRemark.c_str());
						}
						else
						{
							// display remark message
							SendMessage (TmpWindow.hStatusBar, WM_SETTEXT, 0,
								(LPARAM) TmpWindow.sStatusBar.c_str());
						}
					}

					// save old value
					sBGItemOldValue = TmpItem.sReturn;
					bBGItemOldValue = TmpItem.bReturn;

					return TRUE;
				}

				case BN_CLICKED:
				{
					// OK button is clic
					if (TmpItem.sType == BG_ITEM_BUTTON && TmpItem.sButton == "OK")
					{
						// get window structure
						TmpWindow = BG_WindowGet (sFormName);

						TmpWindow.sReturn = "OK";

						// save window structure
						BGWindow [sFormName] = TmpWindow;

						// send close window command
						SendMessage (hBGCurWnd, WM_SYSCOMMAND, SC_CLOSE, 0);

						return TRUE;
					}

					// CANCEL button is clic
					if (TmpItem.sType == BG_ITEM_BUTTON && TmpItem.sButton == "CANCEL")
					{
						// get window structure
						TmpWindow = BG_WindowGet (sFormName);

						TmpWindow.sReturn = "CANCEL";

						// save window structure
						BGWindow [sFormName] = TmpWindow;

						// send close window command
						SendMessage (hBGCurWnd, WM_SYSCOMMAND, SC_CLOSE, 0);

						return TRUE;
					}

					// CHECKBOX change
					if (TmpItem.sType == BG_ITEM_CHECKSYS)
					{
						// call user procedure
						if (TmpWindow.pfFormProc != NULL)
						{
							iError = SendMessage (TmpItem.hHandle, BM_GETCHECK, 0, 0);
							if (iError == BST_CHECKED)
								TmpItem.bReturn = true;
							else
								TmpItem.bReturn = false;

							// save item structure
							BGItem [sFormName + "." + sItemName] = TmpItem;

							iError = TmpWindow.pfFormProc (sFormName, sItemName, BG_MSG_ITEMCHANGE,
											hBGCurWnd, message, wParam, lParam);
							// stop procedure, if user has process the message
							if (iError != 0)
								return iError;

							return TRUE;
						}
					}

					// other BUTTON is clic
					if (TmpItem.sType == BG_ITEM_BUTTON)
					{
						// call user procedure
						if (TmpWindow.pfFormProc != NULL)
						{
							iError = TmpWindow.pfFormProc (sFormName, sItemName, BG_MSG_ITEMCLIC,
											hBGCurWnd, message, wParam, lParam);
							// stop procedure, if user has process the message
							if (iError != 0)
								return iError;

							return TRUE;
						}
					}
				}
			}
		}

		// system menu or close,minimize,maximize
		case WM_SYSCOMMAND:
		{
			switch (wParam)
			{
				// message from keyboard (not documented !!!)
				case 1:
				{
					// enter key is press
					if (GetKeyState (VK_RETURN))
					{
						// goto next item
						PostMessage (hBGCurWnd, WM_NEXTDLGCTL, FALSE, 0);
					}
					
					return TRUE;
				}

				// close form
				case SC_CLOSE:
				{
					// get window structure
					TmpWindow = BG_WindowGet (sFormName);

					if (TmpWindow.sReturn == "")
						TmpWindow.sReturn = "CLOSE";

					// save window structure
					BGWindow [sFormName] = TmpWindow;

					// destroy status bar
					if (TmpWindow.bStatusBar)
					{
						iError = DestroyWindow (TmpWindow.hStatusBar);
						if (iError == 0)
							BG_ErrorDisplay (BG_ERR_W32DESTROYWINDOW);
					}


					// close dialog form
					if (TmpWindow.bModal)
					{
						iError = EndDialog (hBGCurWnd, 0);
						if (iError == 0)
							BG_ErrorDisplay (BG_ERR_W32ENDDIALOG);
					}
					else
					{
						iError = DestroyWindow (hBGCurWnd);
						if (iError == 0)
							BG_ErrorDisplay (BG_ERR_W32DESTROYWINDOW);
					}

					// reset current form
					BGCurForm = "";

					// delete handle
					DeleteObject (hWindowBgBrush);

					// delete brush for selected form
					for (BGItemIt = BGItem.begin(); BGItemIt != BGItem.end(); BGItemIt++)
					{
						sString = BGItemIt->first;
						TmpItem = BGItemIt->second;
						iTemp = sString.find (".");
						sString = sString.substr (0, iTemp);

						if (sString == sFormName)
						{
							if (TmpItem.hBackBrush != NULL)
								DeleteObject (TmpItem.hBackBrush);
							if (TmpItem.hTextBrush != NULL)
								DeleteObject (TmpItem.hTextBrush);

							// save structure
							BGItem [sFormName + "." + sItemName] = TmpItem;
						}
					}

					return TRUE;
				}

				default:
					return FALSE;

			}
		}
	}

	return FALSE;

}



//-----------------------------------------------------------------
//!    BG_FormItemNullGet : return null form item structure       !
//-----------------------------------------------------------------
//! return : null BG_STRUCT_FORMITEM structure                    !
//-----------------------------------------------------------------
BG_STRUCT_FORMITEM BG_FormItemNullGet ()
{
	BG_STRUCT_FORMITEM TmpItem;

	// no error management

	TmpItem.sType = "";
	TmpItem.sFormat = "";
	TmpItem.iLength = 0;
	TmpItem.iOrder = 0;
	TmpItem.hHandle = NULL;
	TmpItem.sValue = "";
	TmpItem.bNotNull = false;
	TmpItem.bEdit = false;
	TmpItem.bEnable = true;
	TmpItem.bVisible = true;
	TmpItem.bCheck = false;
	TmpItem.iBackColor = 0;
	TmpItem.iTextColor = 0;
	TmpItem.iId = 0;
	TmpItem.iLeft = 0;
	TmpItem.iTop = 0;
	TmpItem.iWidth = 0;
	TmpItem.iHeight = 0;
	TmpItem.sButton = "";
	TmpItem.sRemark = "";
	TmpItem.sRetType = "";
	TmpItem.iTextHPos = -1;
	TmpItem.iTextVPos = -1;
	TmpItem.sFtName = "";
	TmpItem.iFtSize = 0;
	TmpItem.bFtBold = false;
	TmpItem.bFtItalic = false;
	TmpItem.bFtUline = false;
	TmpItem.iBrdSize = 0;
	TmpItem.iBrdColor = -1;
	TmpItem.sBrdStyle = "";
	TmpItem.sParam = "";
	TmpItem.bReturn = false;
	TmpItem.sReturn = "";
	TmpItem.curReturn.setnull ();
	TmpItem.dtReturn.setnull ();

	return TmpItem;

}



//-----------------------------------------------------------------
//!    BG_FormItemFontSet : Change font for form item             !
//-----------------------------------------------------------------
//! hBGdevice : device context                                    !
//! sBGFontName : name of the font to use                         !
//!				   if == "" : keep last font                      !
//! iBGFontHeight : height of the font                            !
//! iBGBold : bold           \   ( false = no                     !
//! iBGItalic : italic        =- ( true = yes                     !
//! iBGUnderline: underline  /                                    !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_FormItemFontSet (const HDC hBGdevice, const string sBGFontName, const int iBGFontHeight,
				const bool bBGBold, const bool bBGItalic, const bool bBGUnderline)
				// bBGBold = false, bBGItalic = false, bBGUnderline = false
{
	int iTmpWeight;
	HFONT hfont;

	int iHFont;
	int iError;
	HGDIOBJ hError;


	// for error management
	BG_ErrorSet ("", "BG_FormItemFontSet");
	
	// modify BG bold,italic,underline values
	if (!bBGBold)
		iTmpWeight = FW_REGULAR;
	else
		iTmpWeight = FW_BOLD;

	// compute font size
	iHFont = -MulDiv (iBGFontHeight, GetDeviceCaps (hBGdevice, LOGPIXELSY), 72);

	// set the new font
	hfont = CreateFont(
				iHFont,									// height of font
				0,										// average character width
				0,										// angle of escapement
				0,										// base-line orientation angle
				iTmpWeight,								// font weight
				bBGItalic,								// italic attribute option
				bBGUnderline,							// underline attribute option
				false,									// strikeout attribute option
				ANSI_CHARSET,							// character set identifier
				OUT_DEFAULT_PRECIS,						// output precision
				CLIP_DEFAULT_PRECIS,					// clipping precision
				DEFAULT_QUALITY,						// output quality
				DEFAULT_PITCH | FF_SWISS,				// pitch and family
				sBGFontName.data());					// typeface name

	if (hfont == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32CREATEFONT);
		return BG_ERR_W32CREATEFONT;
	}

	// set and delete font object
	hError = SelectObject (hBGdevice, hfont);
	if (hError == NULL)
	{
		BG_ErrorDisplay (BG_ERR_W32DELETEOBJECT);
		return BG_ERR_W32DELETEOBJECT;
	}

	iError = DeleteObject (hfont);
	if (iError == 0)
	{
		BG_ErrorDisplay (BG_ERR_W32DELETEOBJECT);
		return BG_ERR_W32DELETEOBJECT;
	}

	return 0;

}



//-----------------------------------------------------------------
//!    BG_RectangleDraw : draw one rectangle                      !
//-----------------------------------------------------------------
//! hBGdevice : device context                                    !
//! iBGLeft : left position                                       !
//! iBGTop : top position                                         !
//! iBGRight : right position                                     !
//! iBGBottom : bottom position                                   !
//! sBGStyle : style of rectangle (SOLID,3DIN,3DOUT)              !
//! iBGLineSize : size of border line                             !
//! iBGColor : color of rectangle                                 !
//! iBGBackColor : backgroung color (-1 = transparent)            !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_RectangleDraw (const HDC hBGdevice, const int iBGLeft, const int iBGTop,
					  const int iBGRight, const int iBGBottom, const string sBGStyle,
					  const int iBGLineSize, const int iBGColor, const int iBGBackColor)
				// iBGLineSize = 0, iBGColor = 0, iBackColor = -1
{
	RECT rcSize;
	int iLineSize;
	int iHalfSize;
	int iHalf1Size;
	HPEN hpPen;
	HPEN hpOldPen;
	HBRUSH hbBrush;
	HBRUSH hbOldBrush;
	LOGBRUSH lbBrush;
	int iError;


	// no error management

	// save RECT structure
	rcSize.left = iBGLeft;
	rcSize.top = iBGTop;
	rcSize.right = iBGRight;
	rcSize.bottom = iBGBottom;

	// verify line size
	if (iBGLineSize < 0 || iBGLineSize > 16)
		iLineSize = 1;
	else
		iLineSize = iBGLineSize;


	// draw back ground rectangle if not transparent
	if (iBGBackColor != -1)
	{
		// select pen and save old one
		lbBrush.lbStyle = BS_SOLID;
		lbBrush.lbColor = 0; //iBGBackColor;
		lbBrush.lbHatch = 0;
		hpPen = ExtCreatePen (PS_NULL, 0, &lbBrush, 0, NULL);
		if (hpPen == NULL)
			BG_ErrorDisplay (BG_ERR_W32EXTCREATEPEN);
		hpOldPen = (HPEN) SelectObject (hBGdevice, hpPen);

		// select brush and save old one
		hbBrush = CreateSolidBrush (iBGBackColor);
		hbOldBrush = (HBRUSH) SelectObject (hBGdevice, hbBrush);

		// draw back ground rectangle
		iError = Rectangle (hBGdevice, iBGLeft, iBGTop, iBGRight + 1, iBGBottom + 1);
		if (iError == 0)
			BG_ErrorDisplay (BG_ERR_W32RECTANGLE);

		// delete brush and restore old
		DeleteObject (hbBrush);
		SelectObject (hBGdevice, hbOldBrush);

		// delete pen and restore old
		DeleteObject (hpPen);
		SelectObject (hBGdevice, hpOldPen);
	}

	// verify style
	if (sBGStyle == "SOLID")
	{
		// compute rectangle position
		iHalfSize = iLineSize / 2;
		iHalf1Size = (iLineSize + 1) / 2;

		// add line size to left and top
		rcSize.left += iHalfSize;
		rcSize.top += iHalfSize;

		// substract line size to right and bottom
		rcSize.right -= iHalf1Size;
		rcSize.bottom -= iHalf1Size;

		if (iLineSize != 0)
		{
			// select pen and save old
			lbBrush.lbStyle = BS_SOLID;
			lbBrush.lbColor = iBGColor;
			lbBrush.lbHatch = 0;
			hpPen = ExtCreatePen (PS_GEOMETRIC | PS_SOLID | PS_ENDCAP_SQUARE,
									iLineSize, &lbBrush, 0, NULL);
			if (hpPen == NULL)
				BG_ErrorDisplay (BG_ERR_W32EXTCREATEPEN);
			hpOldPen = (HPEN) SelectObject (hBGdevice, hpPen);

			// draw rectangle
			iError = MoveToEx (hBGdevice, rcSize.left, rcSize.top, (LPPOINT) NULL);
			if (iError == 0)
				return BG_ERR_W32MOVETOEX;

			iError = LineTo (hBGdevice, rcSize.right, rcSize.top);
			if (iError == 0)
				return BG_ERR_W32LINETO;
			iError = LineTo (hBGdevice, rcSize.right, rcSize.bottom);
			if (iError == 0)
				return BG_ERR_W32LINETO;
			iError = LineTo (hBGdevice, rcSize.left, rcSize.bottom);
			if (iError == 0)
				return BG_ERR_W32LINETO;
			iError = LineTo (hBGdevice, rcSize.left, rcSize.top);
			if (iError == 0)
				return BG_ERR_W32LINETO;

			// delete pen and restore old
			DeleteObject (hpPen);
			SelectObject (hBGdevice, hpOldPen);
		}
	}
	else if (sBGStyle == "3DIN")
	{
		// draw "3D IN" rectangle
		iError = DrawEdge (hBGdevice, &rcSize, EDGE_SUNKEN, BF_RECT);
		if (iError == 0)
			return BG_ERR_W32DRAWEDGE;
	}
	else if (sBGStyle == "3DOUT")
	{
		// draw "3D OUT" rectangle
		iError = DrawEdge (hBGdevice, &rcSize, EDGE_RAISED, BF_RECT);
		if (iError == 0)
			return BG_ERR_W32DRAWEDGE;
	}
	else
	{
		return BG_ERR_RECTBADSTYLE;
	}


	return 0;

}



//-----------------------------------------------------------------
//!    BG_FormItemGet : return form item structure                !
//-----------------------------------------------------------------
//! sBGFormName : name of the form                                !
//! sBGItemName : name of the item                                !
//!                                                               !
//! return : BG_STRUCT_FORMITEM structure (null if not found)     !
//-----------------------------------------------------------------
BG_STRUCT_FORMITEM BG_FormItemGet (const string sBGFormName, const string sBGItemName)
{
	BG_STRUCT_FORMITEM TmpItem;
	BGItemItDef BGItemIt;

	// no error management

	// search item
	BGItemIt = BGItem.find (sBGFormName + "." + sBGItemName);
	if (BGItemIt != BGItem.end())
		TmpItem = BGItem [sBGFormName + "." + sBGItemName];
	else
		TmpItem = BG_FormItemNullGet ();

	return TmpItem;

}



//-----------------------------------------------------------------
//!    BG_FormItemModify : modify one parameter from one item     !
//-----------------------------------------------------------------
//! sBGFormName : name of the form                                !
//! sBGItemName : name of the item                                !
//! sBGParam : parameter to modify                                !
//! sBGValue : new value of the parameter                         !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_FormItemModify (const string sBGFormName, const string sBGItemName,
					   const string sBGParam, const string sBGValue)
{
	BG_STRUCT_FORMITEM TmpItem;
	BGItemItDef BGItemIt;
	string sParam;
	string sString;
	int iError;


	// no error management


	// search item
	BGItemIt = BGItem.find (sBGFormName + "." + sBGItemName);
	if (BGItemIt != BGItem.end())
		TmpItem = BGItem [sBGFormName + "." + sBGItemName];
	else
		return BG_ERR_FORMITEMNAMEBAD;

	// tranfert into local and upper case string 
	sParam = BG_StringToUpper (sBGParam);


	// change item value

	// background color
	if (sParam == BG_PRM_BACKCOLOR)
	{
		TmpItem.iBackColor = BG_ColorNumGet (sBGValue);
	}
	// text color
	else if (sParam == BG_PRM_TEXTCOLOR)
	{
		TmpItem.iTextColor = BG_ColorNumGet (sBGValue);
	}
	// checkbox is check
	else if (sParam == BG_PRM_CHECK)
	{
		if (BG_StringIsTrue (sBGValue))
			TmpItem.bCheck = true;
		else
			TmpItem.bCheck = false;
	}
	// item is enable
	else if (sParam == BG_PRM_ENABLE)
	{
		if (BG_StringIsFalse (sBGValue))
		{
			TmpItem.bEnable = false;
			EnableWindow (TmpItem.hHandle, false);
		}
		else
		{
			TmpItem.bEnable = true;
			EnableWindow (TmpItem.hHandle, true);
		}
	}
	// item is visible
	else if (sParam == BG_PRM_VISIBLE)
	{
		if (BG_StringIsFalse (sBGValue))
		{
			TmpItem.bVisible = false;
			ShowWindow (TmpItem.hHandle, SW_HIDE);
		}
		else
		{
			TmpItem.bVisible = true;
			ShowWindow (TmpItem.hHandle, SW_SHOW);

			// enable window because unvisible window are disable
			if (TmpItem.bEnable)
				EnableWindow (TmpItem.hHandle, true);
		}
	}
	// default value
	else if (sParam == BG_PRM_VALUE)
	{
		TmpItem.sValue = sBGValue;
	}
	// function of the button
	else if (sParam == BG_PRM_BUTTON)
	{
		TmpItem.sButton = sBGValue;
	}
	// length of the text into editbox
	else if (sParam == BG_PRM_LENGTH)
	{
		TmpItem.iLength = BG_StringToInt (sBGValue);
	}
	// format of text into editbox
	else if (sParam == BG_PRM_FORMAT)
	{
		TmpItem.sFormat = sBGValue;
	}
	// remark of the item
	else if (sParam == BG_PRM_REMARK)
	{
		TmpItem.sRemark = sBGValue;
	}
	// item can or cannot be null
	else if (sParam == BG_PRM_NOTNULL)
	{
		if (BG_StringIsTrue (sBGValue))
			TmpItem.bNotNull = true;
		else
			TmpItem.bNotNull = false;
	}
	// combobox can be edited
	else if (sParam == BG_PRM_EDIT)
	{
		if (BG_StringIsTrue (sBGValue))
			TmpItem.bEdit = true;
		else
			TmpItem.bEdit = false;
	}
	// text horizontal position
	else if (sParam == BG_PRM_TXHPOS)
	{
		sString = BG_StringToUpper (sBGValue);
		if (sString == "CENTER" || sBGValue == "0")
			TmpItem.iTextHPos = 0;
		else if (sString == "RIGHT" || sBGValue == "-1")
			TmpItem.iTextHPos = 1;
		else
			TmpItem.iTextHPos = -1;
	}
	// text vertical position
	else if (sParam == BG_PRM_TXVPOS)
	{
		sString = BG_StringToUpper (sBGValue);
		if (sString == "CENTER" || sBGValue == "0")
			TmpItem.iTextVPos = 0;
		else if (sString == "BOTTOM" || sBGValue == "1")
			TmpItem.iTextVPos = 1;
		else
			TmpItem.iTextVPos = -1;
	}
	// font name for the text
	else if (sParam == BG_PRM_FTNAME)
	{
		TmpItem.sFtName = sBGValue;
	}
	// font size for the text
	else if (sParam == BG_PRM_FTSIZE)
	{
		TmpItem.iFtSize = BG_StringToInt (sBGValue);
	}
	// font is in bold
	else if (sParam == BG_PRM_FTBOLD)
	{
		if (BG_StringIsTrue (sBGValue))
			TmpItem.bFtBold = true;
		else
			TmpItem.bFtBold = false;
	}
	// font is in italic
	else if (sParam == BG_PRM_FTITALIC)
	{
		if (BG_StringIsTrue (sBGValue))
			TmpItem.bFtItalic = true;
		else
			TmpItem.bFtItalic = false;
	}
	// font is in underline
	else if (sParam == BG_PRM_FTULINE)
	{
		if (BG_StringIsTrue (sBGValue))
			TmpItem.bFtUline = true;
		else
			TmpItem.bFtUline = false;
	}
	// solid border size
	else if (sParam == BG_PRM_BRDSIZE)
	{
		TmpItem.iBrdSize = BG_StringToInt (sBGValue);
	}
	// border color
	else if (sParam == BG_PRM_BRDCOL)
	{
		TmpItem.iBrdColor = BG_ColorNumGet (sBGValue);
	}
	// border style
	else if (sParam == BG_PRM_BRDSTYLE)
	{
		TmpItem.sBrdStyle = sBGValue;
	}
	else
		return BG_ERR_FORMBADPARAM;


	// save item
	BGItem [sBGFormName + "." + sBGItemName] = TmpItem;

	// redraw the item
	iError = BG_FormItemRedraw (sBGFormName, sBGItemName);
	if (iError != 0)
		return iError;


	return 0;

}



//-----------------------------------------------------------------
//!    BG_FormItemRedraw : redraw one item into a form            !
//-----------------------------------------------------------------
//! sBGFormName : name of the form                                !
//! sBGItemName : name of the item                                !
//!                                                               !
//! return : error or 0 if none                                   !
//-----------------------------------------------------------------
int BG_FormItemRedraw (const string sBGFormName, const string sBGItemName)
{
	BG_STRUCT_FORMITEM TmpItem;
	BGItemItDef BGItemIt;
	int iError;

	// no error management


	// search item
	BGItemIt = BGItem.find (sBGFormName + "." + sBGItemName);
	if (BGItemIt != BGItem.end())
		TmpItem = BGItem [sBGFormName + "." + sBGItemName];
	else
		return BG_ERR_FORMITEMNAMEBAD;


	// redraw the item window
	iError = RedrawWindow (TmpItem.hHandle, NULL, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_UPDATENOW);
	if (iError == 0)
		return BG_ERR_W32INVALIDATERECT;


	return 0;

}



//-----------------------------------------------------------------
//!    BG_FormItemNameGet : return item name                      !
//-----------------------------------------------------------------
//! sBGFormName : name of the form                                !
//! iBGId : Id of the form item (return by message loop)          !
//!                                                               !
//! return : string or "" if not found                            !
//-----------------------------------------------------------------
string BG_FormItemNameGet (const string sBGFormName, const int iBGId)
{

	BG_STRUCT_FORMITEM TmpItem;
	BGItemItDef BGItemIt;
	string sFormName;
	string sReturn;
	int iCount;


	// no error management

	// search item by handle
	for (BGItemIt = BGItem.begin(); BGItemIt != BGItem.end(); BGItemIt++)
	{
		// search for form name
		sReturn = BGItemIt->first;
		iCount = sReturn.find (".");
		sFormName = sReturn.substr (0, iCount);

		TmpItem = BGItemIt->second;
		if (sFormName == sBGFormName && TmpItem.iId == iBGId)
			break;
	}

	if (BGItemIt != BGItem.end())
		sReturn = sReturn.substr (iCount + 1);
	else
		sReturn = "";


	return sReturn;

}



//-----------------------------------------------------------------
//!    BG_FormItemNameGet : return item name                      !
//-----------------------------------------------------------------
//! sBGFormName : name of the form                                !
//! hBGHandle : handle of the form item (return by message loop)  !
//!                                                               !
//! return : string or "" if not found                            !
//-----------------------------------------------------------------
string BG_FormItemNameGet (const string sBGFormName, const HWND hBGHandle)
{

	BG_STRUCT_FORMITEM TmpItem;
	BGItemItDef BGItemIt;
	string sFormName;
	string sReturn;
	int iCount;


	// no error management

	// search item by handle
	for (BGItemIt = BGItem.begin(); BGItemIt != BGItem.end(); BGItemIt++)
	{
		// search for form name
		sReturn = BGItemIt->first;
		iCount = sReturn.find (".");
		sFormName = sReturn.substr (0, iCount);

		TmpItem = BGItemIt->second;
		if (sFormName == sBGFormName && TmpItem.hHandle == hBGHandle)
			break;
	}

	if (BGItemIt != BGItem.end())
		sReturn = sReturn.substr (iCount + 1);
	else
		sReturn = "";


	return sReturn;

}



//-----------------------------------------------------------------
//!    BG_FormItemValueGetString : return item value into string  !
//-----------------------------------------------------------------
//! sBGFormName : name of the form                                !
//! sBGItemName : name of the item                                !
//!                                                               !
//! return : string or "" if error                                !
//-----------------------------------------------------------------
string BG_FormItemValueGetString (const string sBGFormName, const string sBGItemName)
{

	BG_STRUCT_FORMITEM TmpItem;
	BGItemItDef BGItemIt;


	// for error management
	BG_ErrorSet ("", "BG_FormItemValueGetString");

	// search item
	BGItemIt = BGItem.find (sBGFormName + "." + sBGItemName);
	if (BGItemIt == BGItem.end())
	{
		BG_ErrorDisplay (BG_ERR_FORMITEMTYPEBAD);
		return "";
	}

	// retreive item structure
	TmpItem = BGItemIt->second;

	// return value if type is compatible with string
	if (TmpItem.sRetType == BG_RET_STRING)
		return TmpItem.sValue;
	else
	{
		BG_ErrorDisplay (BG_ERR_FORMITEMRETBAD);
		return "";
	}

}





//*****************************************************************
//    MEN AT WORK !!!!!!!!
//*****************************************************************

//-----------------------------------------------------------------
//!    BG_StringToUpper : Convert a string to upper case          !
//-----------------------------------------------------------------
//! sBGString : String in lower case box                          !
//!                                                               !
//! return : String in upper case                                 !
//-----------------------------------------------------------------
string BG_StringToUpper (const string sBGString)
{

	char cChar;
	string sUpper = "";
	unsigned int iIndex = 0;

	while (iIndex < sBGString.length())
	{
		cChar = sBGString [iIndex];

		// if cChar between "a-z", transform into "A-Z"
		if (cChar >= 'a' && cChar <= 'z')
			cChar = cChar - 32;

		sUpper = sUpper + cChar;

		iIndex++;
	}

	return sUpper;

}




